#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'weights_init'
__author__ = 'JieYuan'
__mtime__ = '2019/4/17'
"""

from torch.nn import init


class WeightInit(object):

    def __init__(self, layers=['Linear']):
        self.layers = layers

    def __call__(self, model):
        return self._fn(model)

    def _fn(self, model):
        classname = model.__class__.__name__
        # 对layers中包含的层初始化
        if any(map(lambda layer: classname.find(layer) != -1, self.layers)):
            init.xavier_normal_(model.weight.data)
            init.constant_(model.bias.data, 0.0)


if __name__ == '__main__':
    from torch import nn


    class Net(nn.Module):
        pass


    # apply函数会递归地搜索网络内的所有module并把参数表示的函数应用到所有的module上。
    # 对所有的Conv层都初始化权重.
    net = Net()
    net.apply(WeightInit(layers=['Conv']))

    # def weights_init(model, layers):
    #     classname = model.__class__.__name__
    #     # 对layers中包含的层初始化
    #     if any(map(lambda layer: classname.find(layer) != -1, layers)):
    #         init.xavier_normal_(model.weight.data)
    #         init.constant_(model.bias.data, 0.0)
    # from functools import partial
    # weights_init = partial(weights_init, layers=['Conv'])
    # net.apply(weights_init)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : DataIter
# @Time         : 2019-06-21 14:48
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  :

import numpy as np
from tensorflow.python.keras.utils import Sequence
from sklearn.utils import shuffle


class DataIter(Sequence):
    """tf.data.Dataset.from_tensor_slices((X, y)).shuffle(len(X)).batch(128).repeat()
    dataset = tf.data.Dataset.from_tensor_slices((X, y))
    dataset = dataset.shuffle(len(X))
    dataset = dataset.batch(128)
    dataset = dataset.repeat()
    """

    def __init__(self, x, y, batch_size=128, data_processor=lambda *args: args, data_shuffle=False):
        """

        :param x:
        :param y:
        :param batch_size:
        :param data_processor:
        :param data_shuffle:

            keras_model.fit_generator(DataIter(X, y, batch_size=128), epochs=10)
        """
        self.x, self.y = x, y
        self.batch_size = batch_size
        self.data_shuffle = data_shuffle

        assert callable(data_processor) is True
        self.data_processor = data_processor

    def __len__(self):
        return int(np.ceil(len(self.x) / self.batch_size))  # steps

    def __getitem__(self, idx):
        batch_x = self.x[idx * self.batch_size:(idx + 1) * self.batch_size]
        batch_y = self.y[idx * self.batch_size:(idx + 1) * self.batch_size]

        return self.data_processor(batch_x, batch_y)

    # TODO:
    # @abstractmethod
    # def data_processor(self, batch_x, batch_y):
    #     pass

    def on_epoch_end(self):
        """Method called at the end of every epoch.
        """
        if self.data_shuffle:
            self.x, self.y = shuffle(self.x, self.y)
        else:
            pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = '__init__.py'
__author__ = 'JieYuan'
__mtime__ = '19-1-2'
"""

import time

import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import seaborn as sns
from catboost import CatBoostClassifier
from lightgbm import LGBMClassifier
from sklearn.metrics import roc_auc_score
from sklearn.model_selection import StratifiedKFold, RepeatedStratifiedKFold
from statsmodels.api import GLM, families
from xgboost import XGBClassifier

"""
# TODO: cats
https://lightgbm.readthedocs.io/en/latest/Advanced-Topics.html
"""


class OOF(object):
    """Out of flod prediction
    # TODO 支持回归

    lightGBM一个一个地建立节点; XGboost一层一层地建立节点
    https://blog.csdn.net/friyal/article/details/82758532
    Catboost总是使用完全二叉树。它的节点是镜像的(对称树)。Catboost称对称树有利于避免overfit，增加可靠性，并且能大大加速预测等等。
        计算某个category出现的频率，加上超参数，生成新的numerical features
    # https://blog.csdn.net/linxid/article/details/80723811
    """
    _params = {'metric': 'auc',
               'learning_rate': 0.01,
               'n_estimators': 30000,
               'subsample': 0.8,
               'colsample_bytree': 0.8,
               'class_weight': 'balanced',  ##
               'scale_pos_weight': 1,  ##
               'random_state': 2019,
               'verbosity': -1}
    lgb = LGBMClassifier(n_jobs=16, **_params)  # TODO: 常用模型另存为其他模块
    xgb = XGBClassifier()
    cat = CatBoostClassifier(n_estimators=20000, learning_rate=0.05, loss_function='Logloss', eval_metric='AUC',
                             random_state=2019)

    def __init__(self, estimator=None, folds=None, early_stopping_rounds=300, verbose=100):
        self.estimator = self.lgb if estimator is None else estimator  # 指定lgb: metric xgb: eval_metric
        self.folds = folds if folds else StratifiedKFold(5, True, 2019)  # 支持 RepeatedStratifiedKFold
        self.model_type = self.estimator.__repr__()

        self.early_stopping_rounds = early_stopping_rounds
        self.verbose = verbose
        # self.estimator_agrs = self.getfullargspec(self.estimator.fit).args if hasattr(self.estimator, 'fit') else None

    def fit(self, X, y, X_test, feval=None, cat_feats=None, exclude_columns=None, epochs=16, batch_size=128,
            oof2csv=False, plot=False):
        """
        # TODO: Rank 融合
        :param X: 保证索引唯一
        :param y:
        :param X_test:
        :param feval: roc_auc_score(y_true, y_score)
        :param cat_feats: 类别特征索引
        :param exclude_columns:
        仅针对 nn
        :param epochs:
        :param batch_size:
        :return:
        """
        # 判断输入数据转数据框
        if isinstance(y, pd.Series):
            y.reset_index(drop=True, inplace=True)

        if not isinstance(X, pd.DataFrame):
            X = pd.DataFrame(X)
            X_test = pd.DataFrame(X)
        else:
            X.reset_index(drop=True, inplace=True)
            X_test.reset_index(drop=True, inplace=True)

        # oof评估函数
        feval = feval if feval else roc_auc_score

        # 移除不需要的特征
        if exclude_columns:
            feats = X.columns.difference(exclude_columns)
            X, X_test = X[feats], X_test[feats]

        # Score
        if hasattr(feval, '__repr__'):
            score_name = feval.__repr__().split()[1]
        else:
            score_name = None

        # cv num
        if hasattr(self.folds, 'n_splits'):
            num_cv = self.folds.n_splits
        else:
            num_cv = self.folds.cvargs['n_splits'] * self.folds.n_repeats

        # Cross validation model
        # Create arrays and dataframes to store results
        oof_preds = np.zeros(X.shape[0])
        sub_preds = np.zeros((X_test.shape[0], num_cv))
        self.feature_importance_df = pd.DataFrame()

        for n_fold, (train_idx, valid_idx) in enumerate(self.folds.split(X, y), 1):
            print("\n\033[94mFold %s started at %s\033[0m" % (n_fold, time.ctime()))

            X_train, y_train = X.iloc[train_idx], y[train_idx]
            X_valid, y_valid = X.iloc[valid_idx], y[valid_idx]

            if not hasattr(self.estimator, 'fit'):
                print("该算法无fit方法")
                break
            else:
                if 'LGBMClassifier' in self.model_type:
                    eval_set = [(X_train, y_train), (X_valid, y_valid)]
                    self.estimator.fit(X_train, y_train,
                                       eval_set=eval_set,
                                       categorical_feature=cat_feats if cat_feats else 'auto',
                                       eval_metric='auc',
                                       early_stopping_rounds=self.early_stopping_rounds,
                                       verbose=self.verbose)
                elif 'LGBMRegressor' in self.model_type:
                    # reg_objs = ['regression_l1', 'regression_l2', 'huber', 'fair', 'poisson', 'quantile', 'mape', 'gamma', 'tweedie']
                    eval_set = [(X_train, y_train), (X_valid, y_valid)]
                    self.estimator.fit(X_train, y_train,
                                       eval_set=eval_set,
                                       categorical_feature=cat_feats if cat_feats else 'auto',
                                       # eval_metric='l2',
                                       early_stopping_rounds=self.early_stopping_rounds,
                                       verbose=self.verbose)

                elif 'XGBClassifier' in self.model_type:
                    eval_set = [(X_train, y_train), (X_valid, y_valid)]
                    self.estimator.fit(X_train, y_train,
                                       eval_set=eval_set,
                                       eval_metric='auc',
                                       early_stopping_rounds=self.early_stopping_rounds,
                                       verbose=self.verbose)
                elif 'XGBRegressor' in self.model_type:
                    eval_set = [(X_train, y_train), (X_valid, y_valid)]
                    self.estimator.fit(X_train, y_train,
                                       eval_set=eval_set,
                                       # eval_metric='rmse',
                                       early_stopping_rounds=self.early_stopping_rounds,
                                       verbose=self.verbose)

                elif 'CatBoostClassifier' in self.model_type:
                    eval_set = [(X_train, y_train), (X_valid, y_valid)]
                    self.estimator.fit(X_train, y_train,
                                       eval_set=eval_set,
                                       cat_features=cat_feats,
                                       use_best_model=True,
                                       plot=True,
                                       early_stopping_rounds=self.early_stopping_rounds,
                                       verbose=self.verbose)
                elif 'CatBoostRegressor' in self.model_type:
                    eval_set = [(X_train, y_train), (X_valid, y_valid)]
                    self.estimator.fit(X_train, y_train,
                                       eval_set=eval_set,
                                       cat_features=cat_feats,
                                       use_best_model=True,
                                       plot=True,
                                       early_stopping_rounds=self.early_stopping_rounds,
                                       verbose=self.verbose)

                elif 'RGFClassifier' in self.model_type:
                    pass
                elif 'RGFRegressor' in self.model_type:
                    pass

                # https://www.cnblogs.com/flyu6/p/7691106.html
                elif 'KerasClassifier' in self.model_type:
                    eval_set = [(X_train, y_train), (X_valid, y_valid)]
                    self.estimator.fit(X_train, y_train,
                                       epochs=epochs,
                                       batch_size=batch_size,
                                       validation_data=eval_set)
                elif 'KerasRegressor' in self.model_type:
                    eval_set = [(X_train, y_train), (X_valid, y_valid)]
                    self.estimator.fit(X_train, y_train,
                                       epochs=epochs,
                                       batch_size=batch_size,
                                       validation_data=eval_set)

                elif self.model_type == 'GLM':
                    # TODO: 其他模型的支持
                    self.estimator = GLM(y_train, X_train, family=families.Binomial())
                    self.estimator = self.estimator.fit().predict(X)
                else:
                    # sklearn 原生模型
                    print('Sklearn Fitting ...')
                    self.estimator.fit(X_train, y_train)

                # 计算并保存 preds
                # TODO: 多分类需要修改
                if hasattr(self.estimator, 'predict_proba'):
                    oof_preds[valid_idx] = self.estimator.predict_proba(X_valid)[:, 1]
                    sub_preds[:, n_fold - 1] = self.estimator.predict_proba(X_test)[:, 1]
                else:
                    oof_preds[valid_idx] = self.estimator.predict(X_valid)
                    sub_preds[:, n_fold - 1] = self.estimator.predict(X_test)

            if plot and hasattr(self.estimator, 'feature_importances_'):
                fold_importance_df = pd.DataFrame()
                fold_importance_df["feature"] = X.columns
                fold_importance_df["importance"] = self.estimator.feature_importances_
                fold_importance_df["fold"] = n_fold
                self.feature_importance_df = fold_importance_df.append(self.feature_importance_df)

        # 输出需要的结果
        self.oof_preds = oof_preds
        self.sub_preds = sub_preds.mean(1)
        self.sub_preds_rank = pd.DataFrame(sub_preds).rank().mean(1) / sub_preds.shape[0]  # auc work

        try:
            self.score = feval(y, self.oof_preds)
        except Exception as e:
            self.score = 0
            print('Error feval:', e)

        print("\n\033[94mCV Score %s: %s ended at %s\033[0m" % (score_name, self.score, time.ctime()))

        # 保存的普通平均的得分
        if oof2csv:
            pd.Series(np.append(self.oof_preds, self.sub_preds), name='oof') \
                .to_csv('OOF %s %.4f.csv' % (time.ctime(), self.score), index=False)

        # 是否输出特征重要性
        if plot:
            self.feature_importance_df.sort_values(['fold', 'importance'], 0, False, inplace=True)
            self.plot_importances(self.feature_importance_df, len(X.columns))

    def plot_importances(self, df, topk=64):
        """Display/plot feature importance"""
        assert "feature" in df.columns and "importance" in df.columns, '无["feature", "importance"]'

        data = (df[["feature", "importance"]]
                .groupby("feature")
                .mean()
                .reset_index()
                .sort_values("importance", 0, False))[:topk]

        self.feature_importance_df_agg = data
        plt.figure(figsize=(12, topk // 4))
        sns.barplot(x="importance", y="feature", data=data.assign(feature='col_' + data.feature.astype(str)))
        plt.title('Features (avg over folds)')
        plt.tight_layout()
        plt.savefig('importances.png')




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'CountEncoder'
__author__ = 'JieYuan'
__mtime__ = '19-1-10'
"""
from collections import OrderedDict

import pandas as pd
from sklearn.base import BaseEstimator, TransformerMixin


class CountEncoder(BaseEstimator, TransformerMixin):

    def __init__(self, normalize=False, dropna=False):
        self.dropna = dropna
        self.normalize = normalize
        self.mapper = None

    def fit(self, s: pd.Series):
        self.mapper = (s.value_counts(normalize=self.normalize, dropna=self.dropna)
                       .to_dict(OrderedDict))
        return self

    def transform(self, s: pd.Series):
        _ = s.map(self.mapper).fillna(0)  # 不在训练集的补0
        return _


if __name__ == '__main__':
    s = pd.Series(['a', 'a', 'a', 'b', 'b'])
    ce = CountEncoder()
    print(ce.fit_transform(s))
    print(ce.mapper)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'fm_data'
__author__ = 'JieYuan'
__mtime__ = '2019/4/4'
"""
"""
https://www.cnblogs.com/wkang/p/9788012.html
https://blog.csdn.net/songbinxu/article/details/79662665
"""
import os
from collections import defaultdict
from csv import DictReader
import math


class DF2FFM(object):

    def __init__(self):
        pass

    def transform(self, input, output, exclude, excluse):
        pass

train_path = '../input/train.csv'

dont_use = ['MachineIdentifier',
            'Census_FirmwareVersionIdentifier',
            'Census_OEMModelIdentifier',
            'CityIdentifier']

cols_num = []
cols_cat = []
cols_multi_cat = []
# 大值平滑
too_many_vals = ["Census_PrimaryDiskTotalCapacity",
                 "Census_SystemVolumeTotalCapacity",
                 "Census_TotalPhysicalRAM",
                 "Census_InternalPrimaryDiagonalDisplaySizeInInches",
                 "Census_InternalPrimaryDisplayResolutionHorizontal",
                 "Census_InternalPrimaryDisplayResolutionVertical",
                 "Census_InternalBatteryNumberOfCharges"]

categories = [k for k, v in dtypes.items() if k not in dont_use]
categories_index = dict(zip(categories, range(len(categories))))

field_features = defaultdict()

max_val = 1
with open('train.libffm', 'a') as the_file:
    for t, row in enumerate(DictReader(open(train_path))):
        if t % 100000 == 0:
            print(t, len(field_features), max_val)
        label = [row['HasDetections']]
        ffeatures = []

        for field in categories:
            if field == 'HasDetections':
                continue
            feature = row[field]
            if feature == '':
                feature = "unk"
            if field not in num_cols:
                ff = field + '_____' + feature
            else:
                if feature == "unk" or float(feature) == -1:
                    ff = field + '_____' + str(0)
                else:
                    if field in too_many_vals:
                        ff = field + '_____' + str(int(round(math.log(1 + float(feature)))))
                    else:
                        ff = field + '_____' + str(int(round(float(feature))))

            if ff not in field_features:
                if len(field_features) == 0:
                    field_features[ff] = 1
                    max_val += 1
                else:
                    field_features[ff] = max_val + 1
                    max_val += 1

            fnum = field_features[ff]

            ffeatures.append('{}:{}:1'.format(categories_index[field], fnum))
        line = label + ffeatures
        the_file.write('{}\n'.format(' '.join(line)))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = '__init__.py'
__author__ = 'JieYuan'
__mtime__ = '19-1-3'
"""
import numpy as np
from sklearn.cluster import AgglomerativeClustering, DBSCAN, KMeans, MeanShift
from sklearn.mixture import GaussianMixture
from sklearn.preprocessing import StandardScaler

"""
SpectralClustering
import hdbscan 待补充 
"""


class BaselineCluster(object):
    """
    bc = BaselineCluster(X, y)
    new_X = np.column_stack((
            bc.kmeans.transform(X),
            bc.kmeans.predict(X),
            bc.mean_shift.predict(X),
            bc.gaussian_mixture.predict(X),
            bc.dbscan(),
            bc.agglomerative()
        ))
    """

    def __init__(self, X, y, random_state=None):
        self.standard_scaler = StandardScaler()
        self.X = self.standard_scaler.fit_transform(X.astype(float))
        class_weight = dict(enumerate(len(y) / (2 * np.bincount(y))))
        self.n_clusters = len(class_weight)
        self.sample_weight = [class_weight[i] for i in y]
        self.random_state = random_state

    @property
    def kmeans(self):
        """
        model.transform(X) 返回到x,y的距离
        model.predict(X) 返回类中心
        """
        model = KMeans(self.n_clusters, random_state=self.random_state, n_jobs=8)
        model.fit(self.X, sample_weight=self.sample_weight)
        return model

    @property
    def mean_shift(self):
        """
         Mean-shift（即：均值迁移）的基本思想：在数据集中选定一个点，然后以这个点为圆心，r为半径，画一个圆(二维下是圆)，
         求出这个点到所有点的向量的平均值，而圆心与向量均值的和为新的圆心，然后迭代此过程，直到满足一点的条件结束
        :return: model.predict(X) 返回类中心
        """
        model = MeanShift(n_jobs=8, bin_seeding=True)
        model.fit(self.X)
        return model

    @property
    def gaussian_mixture(self):
        """
        :return: model.predict(X) 返回类中心
        """
        model = GaussianMixture(self.n_clusters, random_state=self.random_state)
        model.fit(self.X)
        return model

    def dbscan(self):
        """
        DBSCAN（Density-Based Spatial Clustering of Application with Noise）基于密度的空间聚类算法

        model.predict(X) 返回类中心
        """
        model = DBSCAN(n_jobs=8)
        return model.fit_predict(self.X, sample_weight=self.sample_weight)

    def agglomerative(self):
        """
        Hierarchical Clustering(层次聚类)：就是按照某种方法进行层次分类，直到满足某种条件为止。
        :return: 只有结果没有模型
        """
        model = AgglomerativeClustering(self.n_clusters)
        return model.fit_predict(self.X)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : TextClean
# @Time         : 2019-06-21 18:19
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

from ...pipe import get_module_path

import jieba.posseg as jp


# TODO: 统计特征、emoji


class TextClean(object):

    def __init__(self):
        self._stopwords = self._get_stopwords()

    # def get_words(self, sent, flags=None):
    #     """
    #     :flags mode: ['v', 'vn']
    #     """
    #     for p in jp.cut(sent.lower()):
    #         if p.flag in flags:
    #             yield p.word

    def get_noun(self, doc, with_flag=False):
        for p in jp.cut(doc.lower()):
            if p.word not in self._stopwords:
                if 'n' in p.flag and len(p.word) > 1:  # 长度大于1且非停顿词的名词
                    yield p if with_flag else p.word

    def _get_stopwords(self):
        with open(get_module_path('../../_data/stop_words.txt', __file__), encoding='utf8') as f:
            return set(f.read().split())

# def remove_special_characters(text):
#     tokens = tokenize_text(text)
#     pattern = re.compile('[{}]'.format(re.escape(string.punctuation)))
#     filtered_tokens = filter(None, [re.sub(pattern=pattern, repl="", string=token) for token in tokens])
#     filtered_text = ''.join(filtered_tokens)
#     return filtered_text
#
#
# def normalize_corpus(corpus):
#     normalized_corpus = []
#     for text in corpus:
#         text = remove_special_characters(text)
#         text = remove_stopwords(text)
#         normalized_corpus.append(text)
#     return normalized_corpus



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'opt'
__author__ = 'JieYuan'
__mtime__ = '19-1-3'
"""

"""optuna
https://www.kaggle.com/canonwu/save-intermediate-results-for-stacking/comments

/https://www.cnblogs.com/gczr/p/7156270.html

https://github.com/bamine/Kaggle-stuff/blob/master/otto/hyperopt_xgboost.py#L39
"""

import warnings

warnings.filterwarnings('ignore')
import pandas as pd
import lightgbm as lgb
from bayes_opt import BayesianOptimization
from bayes_opt.observer import JSONLogger
from bayes_opt.event import Events


class BayesOptLGB(object):
    def __init__(self, X, y, metric='auc', objective='binary', categorical_feature='auto', fix_params={}, topk=10,
                 n_jobs=8, opt_seed=None):
        """
        :param X:
        :param y:
        :param metric:
        :param objective: 'regression' , 'binary' or 'multiclass'
        :param categorical_feature:
        :param fix_params:
        :param topk:
        :param opt_seed:
        """

        self.data = lgb.Dataset(X, y, categorical_feature=categorical_feature, silent=True, free_raw_data=False)
        self.metric = metric
        self.objective = objective
        self.topk = topk

        self.fix_params = fix_params
        self.n_jobs = n_jobs
        self.opt_seed = opt_seed

        self.params_ls = []
        self.params_ls_sk = []
        self.params_best = {}
        self.params_best_sk = {}
        self.params_opt_df = None

        self._iter_ls = []

        if self.fix_params:
            print('Fixed Params: \033[94m%s\033[0m\n' % self.fix_params)  # {'min_child_weight': 0.001}

    @property
    def best_model(self):
        if self.params_best:
            return lgb.train(train_set=self.data, **self.params_best)
        else:
            print('\033[94m%s\033[0m\n' % "Please Run !")

    def run(self, n_iter=5, save_log=False):
        BoParams = {
            'num_leaves': (2 ** 4, 2 ** 8),
            'min_split_gain': (0.01, 1),
            'min_child_weight': (0, 0.01),
            'min_child_samples': (8, 32),
            'subsample': (0.6, 1),
            'colsample_bytree': (0.6, 1),
            'reg_alpha': (0, 1),
            'reg_lambda': (0, 1),
        }
        optimizer = BayesianOptimization(self.__evaluator, BoParams, random_state=self.opt_seed)
        if save_log:
            logger = JSONLogger(path="./opt_lgb_logs.json")
            optimizer.subscribe(Events.OPTMIZATION_STEP, logger)

        if self.fix_params:
            optimizer.set_bounds({k: (v, v) for k, v in self.fix_params.items()})

        # optimizer.probe(
        #     {'num_leaves': 2 ** 7 - 1,
        #      'min_split_gain': 0,
        #      'min_child_weight': 0.001,
        #      'min_child_samples': 6,
        #      'subsample': 0.8,
        #      'colsample_bytree': 0.8,
        #      'reg_alpha': 0.01,
        #      'reg_lambda': 1})

        gp_params = {"alpha": 1e-5, "n_restarts_optimizer": 3}
        optimizer.maximize(init_points=3, n_iter=n_iter, acq='ucb', kappa=2.576, xi=0.0, **gp_params)

        self.__get_params(optimizer)

    def __evaluator(self, num_leaves, min_split_gain, min_child_weight, min_child_samples, subsample, colsample_bytree,
                    reg_alpha, reg_lambda):
        self.__params_sk = dict(
            metric=self.metric,
            boosting_type='gbdt',
            objective=self.objective,
            max_depth=-1,
            num_leaves=int(num_leaves),  # 太大会内存泄漏
            learning_rate=0.01,
            min_split_gain=min_split_gain,
            min_child_weight=min_child_weight,
            min_child_samples=int(min_child_samples),
            subsample=subsample,
            subsample_freq=8,
            colsample_bytree=colsample_bytree,
            reg_alpha=reg_alpha,
            reg_lambda=reg_lambda,
            scale_pos_weight=1,
            random_state=0,
            verbosity=-1,
            n_jobs=self.n_jobs
        )
        params = self.__params_sk.copy()
        # params['verbosity'] = -1
        params['metric'] = self.metric

        _ = lgb.cv(params,
                   self.data,
                   num_boost_round=3000,
                   nfold=5,
                   early_stopping_rounds=100,
                   stratified=False if 'reg' in self.objective else True,
                   show_stdv=False)['%s-mean' % self.metric]

        self._iter_ls.append(len(_))
        return -_[-1] if 'reg' in self.objective else _[-1]

    def __get_params(self, optimizer):
        self.params_opt_df = (
            pd.concat([pd.DataFrame({'iter': self._iter_ls}), pd.DataFrame(optimizer.res)], 1)
                .sort_values('target', ascending=False)
                .reset_index(drop=True)[:self.topk])

        for _, (i, _, p) in self.params_opt_df.iterrows():
            params_sk = {**self.__params_sk, **p, **{'n_estimators': i}}

            params_sk['num_leaves'] = int(params_sk['num_leaves'])
            params_sk['min_child_samples'] = int(params_sk['min_child_samples'])
            params_sk = {k: float('%.3f' % v) if isinstance(v, float) else v for k, v in params_sk.items()}
            self.params_ls_sk.append(params_sk)

            params = params_sk.copy()
            num_boost_round = params.pop('n_estimators')
            self.params_ls.append({'params': params, 'num_boost_round': num_boost_round})

        self.params_best = self.params_ls[0]
        self.params_best_sk = self.params_ls_sk[0]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'word_cloud'
__author__ = 'JieYuan'
__mtime__ = '19-3-1'
"""
import random
from pyecharts import charts


class WordCloud(object):

    def __init__(self, data_pair, shape=None, width='900px', height='500px'):
        """
        ['circle', 'cardioid', 'diamond', 'triangle-forward', 'triangle', 'pentagon', 'star']
        wc.render()
        wc.render_notebook()

        """
        self._shapes = ['circle', 'cardioid', 'diamond', 'triangle-forward', 'triangle', 'pentagon', 'star']
        self.data_pair = data_pair
        self.shape = shape if shape else random.choice(self._shapes)
        self.width = width
        self.height = height

    @property
    def wc(self):
        wc = charts.WordCloud()
        wc.add("WordCloud", data_pair=self.data_pair, shape=self.shape)
        wc.width = self.width
        wc.height = self.height
        return wc


if __name__ == '__main__':
    pairs = [('中国', 33),
             ('苹果', 24),
             ('奚梦瑶', 20),
             ('美国', 16),
             ('特朗普', 16),
             ('何猷君', 15),
             ('戛纳', 13),
             ('红毯', 12),
             ('iPhone', 12),
             ('车队', 9),
             ('车祸', 9),
             ('优衣', 9),
             ('信息', 9),
             ('李亚鹏', 9),
             ('恋情', 9),
             ('任素', 9),
             ('男孩', 9),
             ('亚洲', 8),
             ('孩子', 8),
             ('大学生', 8)]

    WordCloud(pairs).wc.render_notebook()
    WordCloud(pairs).wc.render()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : demo
# @Time         : 2019-06-23 21:36
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


class A:

    def __init__(self, a=1, b=2):
        self.a = a
        self.b = b

        print(self.a + self.b)

    @property
    def f(self):
        yield 111111


class B(A):

    def __init__(self, a=100000, **kwargs):
        self.a = a+11111
        super().__init__(**kwargs)

        print(self.a)

        pass


if __name__ == '__main__':
    print(B(a=1,b=111))





#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : xgb_deploy
# @Time         : 2019-07-03 14:40
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


"""

# spark 模型可导出 py能调用的模型
val nativeModelPath = "/tmp/nativeModel"
xgbClassificationModel.nativeBooster.saveModel(nativeModelPath)

import xgboost as xgb
bst = xgb.Booster({'nthread': 4})
bst.load_model(nativeModelPath)
"""


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'estimator_params_bounds'
__author__ = 'JieYuan'
__mtime__ = '19-3-19'
"""
from sklearn.svm import SVC
from sklearn.ensemble import RandomForestClassifier as RFC


class ParamsBounds(object):
    """https://github.com/Jie-Yuan/optuna/blob/master/examples/lightgbm_simple.py"""

    def __init__(self):
        pass

    @property
    def lgb(self):
        """https://www.cnblogs.com/wzdLY/p/9867719.html"""
        # TODO: 其他 boosting_type

        pbounds = {
            'learning_rate': (0.001, 0.1),
            'num_leaves': (2 ** 2, 2 ** 8),
            'min_split_gain': (0.01, 1),
            'min_child_weight': (0.001, 100),
            'min_child_samples': (16, 128),
            'subsample': (0.1, 1),
            'colsample_bytree': (0.01, 1),
            'reg_alpha': (0.01, 10),
            'reg_lambda': (0.01, 10)
        }
        return pbounds


    """https://github.com/fmfn/BayesianOptimization/blob/master/examples/sklearn_example.py"""
    @property
    def svc(self):
        estimator = SVC(random_state=123456)
        pbounds = {"C": (0.001, 100), "gamma": (0.0001, 0.1)}

        return estimator, pbounds

    @property
    def rfc(self):
        estimator = RFC(random_state=123456, n_jobs=-1)
        pbounds = {
            "criterion": ('gini', 'entropy'),
            "n_estimators": (32, 256),
            "min_samples_split": (2, 25),
            "max_features": (0.01, 1),
        }
        return estimator, pbounds





#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'timer'
__author__ = 'JieYuan'
__mtime__ = '19-1-14'
"""
import time
from contextlib import contextmanager

from .. import cprint


@contextmanager
def timer(task_name="timer"):
    # a timer cm from https://www.kaggle.com/lopuhin/mercari-golf-0-3875-cv-in-75-loc-1900-s
    print('\n')
    cprint(">>> {} started".format(task_name))
    t0 = time.time()
    yield
    cprint(">>> {} done in {:.0f} seconds".format(task_name, time.time() - t0))


if __name__ == '__main__':
    with timer() as t:
        print('xxx')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : Features
# @Time         : 2019-07-26 10:04
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

import featuretools as ft
from featuretools.selection import remove_low_information_features


class Features(object):
    """demo
    假如无时间列
    一拆多
    """

    def __init__(self, sparse_feats=None, dense_feats=None):
        self.sparse_feats = sparse_feats
        self.dense_feats = dense_feats
        self.es = ft.EntitySet(id='MAIN')

    def get_feats(self, df):
        self.es.entity_from_dataframe('df', df.copy(), index='ID')

        print("把类别当做ID拆成新表")
        # 这只是单key聚合
        for v in self.es['df'].variables:
            if v.dtype == 'categorical':
                self.es.normalize_entity('df', f'df_{v.name}', v.name)

        # todo: 多key聚合（如果不支持，先组合成单key）或者 获得 重要类别的子集组合
        # 多类别交叉得到更细的分组统计信息

        self.es.plot()
        df_feats, _ = ft.dfs(entityset=self.es, target_entity='df', verbose=1, max_depth=3, n_jobs=3)
        return df_feats

    def _subsets(self, items, min_len=2, max_len=None):
        N = len(items)
        if max_len is None:
            max_len = N
        else:
            max_len = min(max_len, N)
        for i in range(2 ** N):  # 子集的个数
            combo = []
            for j in range(N):  # 用来判断二进制数的下标为j的位置的数是否为1
                if (i >> j) % 2:
                    combo.append(items[j])
            if min_len <= len(combo) <= max_len:
                yield combo



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'plot_cat'
__author__ = 'JieYuan'
__mtime__ = '2019-04-26'
"""
import seaborn as sns
import matplotlib.pyplot as plt


def plot_count(df, cat='rentType', figsize=(9, 4)):
    """饼图+计数直方图"""
    f, ax = plt.subplots(1, 2, figsize=figsize)
    df[cat].value_counts().plot.pie(autopct='%1.2f%%', ax=ax[0], shadow=True)
    sns.countplot(cat, data=df, ax=ax[1])
    for _ax in ax:
        _ax.set_title(cat)
    plt.show()



def distplot(data, x, y, figsize=(20, 8)):
    """
    g = sns.FacetGrid(train, col='rentType')
    g.map(sns.distplot, 'tradeMoney', bins=20)
    g.map(plt.hist, 'tradeMoney', bins=20)
    """
    cats = data[x].unique()
    f, ax = plt.subplots(1, len(cats), sharey=True, figsize=figsize)
    for cat, _ax in zip(cats, ax):
        _ax.set_title(cat)
        sns.distplot(data[y][data[x] == cat], ax=_ax)
    plt.show()

#
# # sns.heatmap(data.corr(),annot=True,cmap='RdYlGn',linewidths=0.2) #data.corr()-->correlation matrix
# # fig=plt.gcf()
# # fig.set_size_inches(10,8)
# # plt.show()
# # sns.heatmap(train.astype(float).corr(),linewidths=0.1,vmax=1.0,
# #             square=True, cmap=colormap, linecolor='white', annot=True)
#
#
# pd.crosstab([train.houseFloor, train.rentType], train.houseDecoration, margins=True) \
#     .style.background_gradient(cmap='summer_r')
#
# # grid = sns.FacetGrid(train_df, col='Pclass', hue='Survived')
# grid = sns.FacetGrid(train, col='houseFloor', row='rentType', aspect=1.6)
# grid.map(plt.hist, 'tradeMoney', alpha=.5, bins=20)
# grid.add_legend();


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'functions'
__author__ = 'JieYuan'
__mtime__ = '19-1-15'
"""
import numpy as np


class Funcs(object):
    """x: pd.Series"""

    def __init__(self):
        self.num = ['q1', 'q3', 'iqr', 'kurt', 'cv', 'p2p']
        self.cat = ['mode', 'nunique_perc']

    @property
    def num_funcs(self):
        return [self.__getattribute__(func_name) for func_name in self.num]

    @property
    def cat_funcs(self):
        return [self.__getattribute__(func_name) for func_name in self.cat]

    # cat funcs
    def mode(self, x):
        return x.value_counts().index[0]

    def nunique_perc(self, x):
        return x.nunique() / x.count()

    # num funcs
    def q1(self, x):
        return x.quantile(0.25)

    def q3(self, x):
        return x.quantile(0.75)

    def iqr(self, x):
        return x.quantile(0.75) - x.quantile(0.25)

    def p2p(self, x):
        return np.ptp(x)

    def kurt(self, x):
        return x.kurt()

    def cv(self, x):
        return x.std() / (x.mean() + 1e-8)  # 变异系数

    def count_nonzero(self, x):
        return np.count_nonzero(x)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-cloudml.
# @File         : get_embedding_layer
# @Time         : 2019-06-21 16:25
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  :
from tensorflow.python.keras.layers import Embedding

def get_keras_embedding(maxlen, weights=None, trainable=False):
    """

    :param maxlen: 输入序列的长度，当它是固定的时。如果你需要连接 Flatten 和 Dense层，则这个参数是必须的（没有它，dense层的输出尺寸就无法计算）
    :param weights: vectors
    :param trainable:
    :return:
    """
    m, n = weights.shape  # 词数/类别数 * 输出向量维度
    layer = Embedding(input_dim=m,
                      output_dim=n,
                      weights=[weights] if weights else None,
                      trainable=trainable,
                      input_length=maxlen)

    return layer



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'VecFeats'
__author__ = 'JieYuan'
__mtime__ = '19-2-18'
"""

from sklearn.feature_extraction.text import CountVectorizer, TfidfVectorizer


class VecFeats(object):

    def __init__(self, df, cat_feats, vectorizer=TfidfVectorizer(ngram_range=(1, 1), max_features=1000)):
        self.df = df
        self.cat_feats = cat_feats
        self.vectorizer = vectorizer

    def get_vectors(self):
        return self.vectorizer.fit_transform(self.corpus)

    @property
    def corpus(self):
        df = self.df[self.cat_feats]
        for idx, feat in enumerate(self.cat_feats):
            df[feat] = '%s_' % idx + df[feat].astype(str)
        return df.apply(lambda x: ' '.join(x), 1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'download_file'
__author__ = 'JieYuan'
__mtime__ = '18-12-27'
"""
import pickle
import requests
from tqdm import tqdm
from sklearn.externals import joblib


class DownloadFile(object):
    """方便远程部署模型"""

    def get(self, url, mode='sklearn'):
        self.__get_file(url)
        if mode == 'sklearn':
            obj = joblib.load('./temp.txt')
        elif mode == 'pickle':
            with open('./temp.txt', 'rb') as f:
                obj = pickle.load(f)
        else:
            print("please read with open('./temp.txt') !!!")
            obj = "./temp.txt"
        return obj

    def __get_file(self, url, tempfile='./temp.txt'):
        try:
            res = requests.get(url)
            res.raise_for_status()
            with open(tempfile, 'wb') as f:
                for lines in tqdm(res.iter_content(100000), 'Loading'):
                    f.write(lines)
        except:
            print('please input correct url !!!')



# -*- coding: utf-8 -*-
"""
__title__ = 'DataFrameSummary'
__author__ = 'JieYuan'
__mtime__ = '18-11-6'
"""


from six import string_types

from collections import OrderedDict

import numpy as np
import pandas as pd
from pandas.api import types
from concurrent.futures import ProcessPoolExecutor


class DataFrameSummary(object):
    ALL = 'ALL'
    INCLUDE = 'INCLUDE'
    EXCLUDE = 'EXCLUDE'

    TYPE_BOOL = 'bool'
    TYPE_NUMERIC = 'numeric'
    TYPE_DATE = 'date'
    TYPE_CATEGORICAL = 'categorical'
    TYPE_CONSTANT = 'constant'
    TYPE_UNIQUE = 'unique'

    def __init__(self, df):
        self.df = df
        self.length = len(df)
        self.columns_stats = self._get_stats()
        self.corr = df.corr()

    def __getitem__(self, column):
        if isinstance(column, str) and self._clean_column(column):
            return self._get_column_summary(column)

        if isinstance(column, int) and column < self.df.shape[1]:
            return self._get_column_summary(self.df.columns[column])

        if isinstance(column, (tuple, list)):
            error_keys = [k for k in column if not self._clean_column(k)]
            if len(error_keys) > 0:
                raise KeyError(', '.join(error_keys))
            return self.df[list(column)].values

        if isinstance(column, pd.Index):
            error_keys = [
                k for k in column.values if not self._clean_column(k)]
            if len(error_keys) > 0:
                raise KeyError(', '.join(error_keys))
            return self.df[column].values

        if isinstance(column, np.ndarray):
            error_keys = [k for k in column if not self._clean_column(k)]
            if len(error_keys) > 0:
                raise KeyError(', '.join(error_keys))
            return self.df[column].values

        raise KeyError(column)

    @property
    def columns_types(self):
        return pd.value_counts(self.columns_stats.loc['types'])

    def summary(self):
        return pd.concat([self.df.describe(), self.columns_stats], sort=True)[self.df.columns]

    def summary_plus(self):
        with ProcessPoolExecutor(min(7, self.df.shape[1])) as pool:
            _ = pool.map(self._get_column_summary, self.df.columns)
        return pd.concat(_, 1, sort=True)

    @staticmethod
    def _number_format(x):
        eps = 0.000000001
        num_format = '{0:,.0f}' if abs(int(x) - x) < eps else '{0:,.2f}'
        return num_format.format(x)

    @classmethod
    def _percent(cls, x):
        x = cls._number_format(100 * x)
        return '{}%'.format(x)

    def _clean_column(self, column):
        if not isinstance(column, (int, string_types)):
            raise ValueError('{} is not a valid column'.format(column))
        return column in self.df.columns

    def _get_stats(self):
        counts = self.df.count()
        counts.name = 'counts'
        uniques = self._get_uniques()
        missing = self._get_missing(counts)
        stats = pd.concat([counts, uniques, missing], axis=1, sort=True)

        # settings types
        stats['types'] = ''
        columns_info = self._get_columns_info(stats)
        for ctype, columns in columns_info.items():
            stats.loc[columns, 'types'] = ctype
        return stats.transpose()[self.df.columns]

    def _get_uniques(self):
        return pd.Series(dict((c, self.df[c].nunique()) for c in self.df.columns), name='uniques')

    def _get_missing(self, counts):
        count = self.length - counts
        count.name = 'missing'
        perc = (count / self.length).apply(self._percent)
        perc.name = 'missing_perc'
        return pd.concat([count, perc], axis=1, sort=True)

    def _get_columns_info(self, stats):
        column_info = {}
        column_info[self.TYPE_CONSTANT] = stats['uniques'][stats['uniques'] == 1].index
        column_info[self.TYPE_BOOL] = stats['uniques'][stats['uniques'] == 2].index
        rest_columns = self.get_columns(self.df,
                                        self.EXCLUDE,
                                        column_info['constant'].union(column_info['bool']))
        column_info[self.TYPE_NUMERIC] = pd.Index([c for c in rest_columns
                                                   if types.is_numeric_dtype(self.df[c])])
        rest_columns = self.get_columns(
            self.df[rest_columns], self.EXCLUDE, column_info['numeric'])
        column_info[self.TYPE_DATE] = pd.Index([c for c in rest_columns
                                                if types.is_datetime64_dtype(self.df[c])])
        rest_columns = self.get_columns(
            self.df[rest_columns], self.EXCLUDE, column_info['date'])
        unique_columns = stats['uniques'][rest_columns] == stats['counts'][rest_columns]
        column_info[self.TYPE_UNIQUE] = stats['uniques'][rest_columns][unique_columns].index
        column_info[self.TYPE_CATEGORICAL] = stats['uniques'][rest_columns][~unique_columns].index
        return column_info

    """ Column summaries """

    def _get_deviation_of_mean(self, series, multiplier=3):
        """
        Returns count of values deviating of the mean, i.e. larger than `multiplier` * `std`.
        :type series:
        :param multiplier:
        :return:
        """
        capped_series = np.minimum(
            series, series.mean() + multiplier * series.std())
        count = pd.value_counts(series != capped_series)
        count = count[True] if True in count else 0
        perc = self._percent(count / self.length)
        return count, perc

    def _get_median_absolute_deviation(self, series, multiplier=3):
        """
        Returns count of values larger than `multiplier` * `mad`
        :type series:
        :param multiplier:
        :return (array):
        """
        capped_series = np.minimum(
            series, series.median() + multiplier * series.mad())
        count = pd.value_counts(series != capped_series)
        count = count[True] if True in count else 0
        perc = self._percent(count / self.length)
        return count, perc

    def _get_top_correlations(self, column, threshold=0.65, top=3):
        column_corr = np.fabs(self.corr[column].drop(column)).sort_values(ascending=False,
                                                                          inplace=False)
        top_corr = column_corr[(column_corr > threshold)][:top].index
        correlations = self.corr[column][top_corr].to_dict()
        return '｜'.join('{}:{:.2f}'.format(col, val) for col, val in correlations.items())

    def _get_numeric_summary(self, column, plot=True):
        series = self.df[column]

        if plot:
            try:
                series.hist()
            except ImportError:
                pass

        stats = OrderedDict()
        stats['mean'] = series.mean()
        stats['std'] = series.std()
        stats['variance'] = series.var()
        stats['min'] = series.min()
        stats['max'] = series.max()

        for x in np.array([0.05, 0.25, 0.5, 0.75, 0.95]):
            stats[self._percent(x)] = series.quantile(x)

        stats['iqr'] = stats['75%'] - stats['25%']
        stats['kurtosis'] = series.kurt()
        stats['skewness'] = series.skew()
        stats['sum'] = series.sum()
        stats['mad'] = series.mad()
        stats['cv'] = stats['std'] / stats['mean'] if stats['mean'] else np.nan
        stats['zeros_num'] = self.length - np.count_nonzero(series)
        stats['zeros_perc'] = self._percent(stats['zeros_num'] / self.length)
        deviation_of_mean, deviation_of_mean_perc = self._get_deviation_of_mean(
            series)
        stats['deviating_of_mean'] = deviation_of_mean
        stats['deviating_of_mean_perc'] = deviation_of_mean_perc
        deviating_of_median, deviating_of_median_perc = self._get_median_absolute_deviation(
            series)
        stats['deviating_of_median'] = deviating_of_median
        stats['deviating_of_median_perc'] = deviating_of_median_perc
        stats['top_correlations'] = self._get_top_correlations(column)
        return pd.concat([pd.Series(stats, name=column),
                          self.columns_stats.loc[:, column]],
                         sort=True)

    def _get_date_summary(self, column):
        series = self.df[column]
        stats = {'min': series.min(), 'max': series.max()}
        stats['range'] = stats['max'] - stats['min']
        return pd.concat([pd.Series(stats, name=column),
                          self.columns_stats.loc[:, column]],
                         sort=True)

    def _get_categorical_summary(self, column):
        series = self.df[column]
        # Only run if at least 1 non-missing value
        value_counts = series.value_counts()
        stats = {
            'top': '{}: {}'.format(value_counts.index[0], value_counts.iloc[0]),
        }
        return pd.concat([pd.Series(stats, name=column),
                          self.columns_stats.loc[:, column]],
                         sort=True)

    def _get_constant_summary(self, column):
        return 'This is a constant value: {}'.format(self.df[column][0])

    def _get_bool_summary(self, column):
        series = self.df[column]

        stats = {}
        for class_name, class_value in dict(series.value_counts()).items():
            stats['"{}" count'.format(class_name)] = '{}'.format(class_value)
            stats['"{}" perc'.format(class_name)] = '{}'.format(
                self._percent(class_value / self.length))

        return pd.concat([pd.Series(stats, name=column),
                          self.columns_stats.loc[:, column]],
                         sort=True)

    def _get_unique_summary(self, column):
        return self.columns_stats.loc[:, column]

    def _get_column_summary(self, column):
        column_type = self.columns_stats.loc['types'][column]
        if column_type == self.TYPE_NUMERIC:
            return self._get_numeric_summary(column)
        if column_type == self.TYPE_CATEGORICAL:
            return self._get_categorical_summary(column)
        if column_type == self.TYPE_BOOL:
            return self._get_bool_summary(column)
        if column_type == self.TYPE_UNIQUE:
            return self._get_unique_summary(column)
        if column_type == self.TYPE_DATE:
            return self._get_date_summary(column)
        if column_type == self.TYPE_CONSTANT:
            return self._get_constant_summary(column)

    def get_columns(self, df, usage, columns=None):
        """
        Returns a `data_frame.columns`.
        :param df: dataframe to select columns from
        :param usage: should be a value from [ALL, INCLUDE, EXCLUDE].
                            this value only makes sense if attr `columns` is also set.
                            otherwise, should be used with default value ALL.
        :param columns: * if `usage` is all, this value is not used.
                        * if `usage` is INCLUDE, the `df` is restricted to the intersection
                          between `columns` and the `df.columns`
                        * if usage is EXCLUDE, returns the `df.columns` excluding these `columns`
        :return: `data_frame` columns, excluding `target_column` and `id_column` if given.
                 `data_frame` columns, including/excluding the `columns` depending on `usage`.
        """
        columns_excluded = pd.Index([])
        columns_included = df.columns

        if usage == self.INCLUDE:
            try:
                columns_included = columns_included.intersection(pd.Index(columns))
            except TypeError:
                pass
        elif usage == self.EXCLUDE:
            try:
                columns_excluded = columns_excluded.union(pd.Index(columns))
            except TypeError:
                pass

        columns_included = columns_included.difference(columns_excluded)
        return columns_included.intersection(df.columns)



#!/usr/bin/env python
# -*- coding= utf-8 -*-
"""
__title__ =  lgb 
__author__ =  JieYuan 
__mtime__ =  19-1-2 
"""

import lightgbm as lgb


class BaselineLGB(object):
    """
    b_lgb = BaselineLGB(X, y, learning_rate=0.01)
    b_lgb.run()
    """

    def __init__(self, X, y, learning_rate=0.01, categorical_feature='auto', metrics='auc', feval=None,
                 objective='binary', scale_pos_weight=1, n_jobs=8, seed=None):
        """
        :param objective:
            Default: 'regression' for LGBMRegressor, 'binary' or 'multiclass' for LGBMClassifier, 'lambdarank' for LGBMRanker.
        :param metrics: string, list of strings or None, optional (default=None)
            binary: 'auc', 'binary_error', 'binary_logloss'
            multiclass: 'multi_error', 'multi_logloss'
            https://lightgbm.readthedocs.io/en/latest/Parameters.html#metric-parameters
        :param feval:
            def feval(y_pred, y_true):
                y_true = y_true.get_label()
                return '1 / (1 + rmse)', 1 /(rmse(y_true, y_pred) + 1), True
        :param scale_pos_weight:
        """
        self.data = lgb.Dataset(X, y, categorical_feature=categorical_feature, free_raw_data=False, weight=None,
                                init_score=None)  # init_score初始分(例如常值/均值/中位数等回归的得分)
        self.objective = objective
        self.metrics = metrics
        self.feval = feval
        self.best_iter = None

        # sklearn params
        self.params_sk = dict(
            metric=self.metrics,
            boosting_type='gbdt',
            objective=objective,
            max_depth=-1,
            num_leaves=17,
            learning_rate=learning_rate,

            min_child_weight=0.001,  # 决定最小叶子节点样本权重和, 孩子节点中最小的样本权重和, 避免过拟合, 如果一个叶子节点的样本权重和小于min_child_weight则拆分过程结束
            min_child_samples=20,  # 一个叶子上数据的最小数量. 可以用来处理过拟合

            subsample=0.8,
            subsample_freq=3,
            colsample_bytree=0.8,

            min_split_gain=0.0,  # 描述分裂的最小 gain, 控制树的有用的分裂
            reg_alpha=0.0,
            reg_lambda=0.0,

            scale_pos_weight=scale_pos_weight,

            random_state=seed,
            n_jobs=n_jobs,
            verbosity=-1  # < 0: Fatal, = 0: Error (Warning), = 1: Info, > 1: Debug
        )
        self.params = self.params_sk.copy()

        if self.objective == 'multiclass':
            self.num_class = len(set(y))
            self.params['objective'] = self.objective
            self.params['num_class'] = self.num_class

    def run(self, return_model=False, nfold=5, early_stopping_rounds=200, verbose_eval=100):

        print("LGB CV ...\n")
        cv_rst = lgb.cv(
            self.params,
            self.data,
            metrics=self.metrics,
            feval=self.feval,
            nfold=nfold,
            num_boost_round=9999,
            stratified=False if 'reg' in self.objective else True,
            early_stopping_rounds=early_stopping_rounds,
            verbose_eval=verbose_eval,
        )

        if isinstance(self.metrics, str):
            _ = cv_rst['%s-mean' % self.metrics]
            self.best_iter = len(_)
        else:
            _ = cv_rst['%s-mean' % self.metrics[0]]
            self.best_iter = len(_)
        print('\nBest Iter: %s' % self.best_iter)
        print('Best Score: %s ' % _[-1])

        self.params_sk['n_estimators'] = self.best_iter

        if return_model:
            print("\nReturning Model ...\n")
            return lgb.train(self.params, self.data, self.best_iter)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : ExplainerTabular
# @Time         : 2019-06-20 14:06
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

# TODO:
"""
https://github.com/microsoft/interpret
https://github.com/h2oai/mli-resources/blob/master/notebooks/dt_surrogate.ipynb
https://github.com/Jie-Yuan/lime
https://eli5.readthedocs.io/en/latest/blackbox/permutation_importance.html
https://github.com/slundberg/shap
"""


class ExplainerTabular(object):

    def __init__(self):
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : pd_set
# @Time         : 2019-06-20 11:27
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

import pandas as pd


def set_pandas(max_rows=256, max_columns=64):
    """
    pd.set_option('display.max_rows', 1024)
    pd.set_option('display.max_columns', 128)
    pd.set_option('max_colwidth', 128)  # 列宽
    # pd.set_option('expand_frame_repr', False)  # 允许换行显示
    """
    pd.set_option('display.max_rows', max_rows)
    pd.set_option('display.max_columns', max_columns)
    pd.set_option('max_colwidth', 128)  # 列宽
    # pd.set_option('expand_frame_repr', False)  # 允许换行显示
    print('Setting Success!')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : ftrl
# @Time         : 2019-09-15 15:55
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


from datetime import datetime
from csv import DictReader
from math import exp, log, sqrt
import gzip
import random
import json
import argparse


class FTRLProximal(object):
    """
    FTRL Proximal engineer project with logistic regression
    Reference:
    https://static.googleusercontent.com/media/research.google.com/zh-CN//pubs/archive/41159.pdf

    """

    def __init__(self, alpha, beta, L1, L2, D,
                 interaction=False, dropout=1.0,
                 dayfeature=True,
                 device_counters=False):

        # parameters
        self.alpha = alpha
        self.beta = beta
        self.L1 = L1
        self.L2 = L2
        self.dayfeature = dayfeature
        self.device_counters = device_counters

        # feature related parameters
        self.D = D
        self.interaction = interaction
        self.dropout = dropout

        # model
        self.n = [0.] * D
        self.z = [0.] * D
        self.w = [0.] * D

    def _indices(self, x):
        '''
        A helper generator that yields the indices in x
        The purpose of this generator is to make the following
        code a bit cleaner when doing feature interaction.
        '''

        for i in x:
            yield i

        if self.interaction:
            D = self.D
            L = len(x)
            for i in range(1, L):  # skip bias term, so we start at 1
                for j in range(i + 1, L):
                    # one-hot encode interactions with hash trick
                    yield abs(hash(str(x[i]) + '_' + str(x[j]))) % D

    def predict(self, x, dropped=None):
        """
        use x and computed weight to get predict
        :param x:
        :param dropped:
        :return:
        """
        # wTx is the inner product of w and x
        wTx = 0.
        for j, i in enumerate(self._indices(x)):

            if dropped is not None and dropped[j]:
                continue

            wTx += self.w[i]

        if dropped is not None:
            wTx /= self.dropout

        # bounded sigmoid function, this is the probability estimation
        return 1. / (1. + exp(-max(min(wTx, 35.), -35.)))

    def update(self, x, y):
        """
        update weight and coordinate learning rate based on x and y
        :param x:
        :param y:
        :return:
        """

        ind = [i for i in self._indices(x)]

        if self.dropout == 1:
            dropped = None
        else:
            dropped = [random.random() > self.dropout for i in range(0, len(ind))]

        p = self.predict(x, dropped)

        # gradient under logloss
        g = p - y

        # update z and n
        for j, i in enumerate(ind):

            # implement dropout as overfitting prevention
            if dropped is not None and dropped[j]:
                continue

            g_i = g * i
            sigma = (sqrt(self.n[i] + g_i * g_i) - sqrt(self.n[i])) / self.alpha
            self.z[i] += g_i - sigma * self.w[i]
            self.n[i] += g_i * g_i

            sign = -1. if self.z[i] < 0 else 1.  # get sign of z[i]

            # build w on the fly using z and n, hence the name - lazy weights -
            if sign * self.z[i] <= self.L1:
                # w[i] vanishes due to L1 regularization
                self.w[i] = 0.
            else:
                # apply prediction time L1, L2 regularization to z and get
                self.w[i] = (sign * self.L1 - self.z[i]) \
                            / ((self.beta + sqrt(self.n[i])) / self.alpha + self.L2)

    def save_model(self, save_file):
        """
        保存weight数据到本地
        :param save_file:
        :return:
        """
        with open(save_file, "w") as f:
            w = {k: v for k, v in enumerate(self.w) if v != 0}
            z = {k: v for k, v in enumerate(self.z) if v != 0}
            n = {k: v for k, v in enumerate(self.n) if v != 0}
            data = {
                'w': w,
                'z': z,
                'n': n
            }
            json.dump(data, f)

    def load_weight(self, model_file, D):
        """
        loada weight data
        :param model_file:
        :return:
        """
        with open(model_file, "r") as f:
            data = json.load(f)
            self.w = data.get('w', [0.] * D)
            self.z = data.get('z', [0.] * D)
            self.n = data.get('n', [0.] * D)

    @staticmethod
    def loss(y, y_pred):
        """
        log loss for LR model
        :param y:
        :param y_pred:
        :return:
        """
        p = max(min(y_pred, 1. - 10e-15), 10e-15)
        return -log(p) if y == 1. else -log(1. - p)


def data(f_train, D, dayfilter=None, dayfeature=True, counters=False):
    ''' GENERATOR: Apply hash-trick to the original csv row
                   and for simplicity, we one-hot-encode everything

        INPUT:
            path: path to training or testing file
            D: the max index that we can hash to

        YIELDS:
            ID: id of the instance, mainly useless
            x: a list of hashed and one-hot-encoded 'indices'
               we only need the index since all values are either 0 or 1
            y: y = 1 if we have a click, else we have y = 0
    '''

    device_ip_counter = {}
    device_id_counter = {}

    for t, row in enumerate(DictReader(f_train)):
        # process id
        ID = row['id']
        del row['id']

        # process clicks
        y = 0.
        if 'click' in row:
            if row['click'] == '1':
                y = 1.
            del row['click']

        # turn hour really into hour, it was originally YYMMDDHH

        date = row['hour'][0:6]
        row['hour'] = row['hour'][6:]

        if dayfilter != None and not date in dayfilter:
            continue

        if dayfeature:
            # extract date
            row['wd'] = str(int(date) % 7)
            row['wd_hour'] = "%s_%s" % (row['wd'], row['hour'])

        if counters:
            d_ip = row['device_ip']
            d_id = row["device_id"]
            try:
                device_ip_counter[d_ip] += 1
                device_id_counter[d_id] += 1
            except KeyError:
                device_ip_counter[d_ip] = 1
                device_id_counter[d_id] = 1
            row["ipc"] = str(min(device_ip_counter[d_ip], 8))
            row["idc"] = str(min(device_id_counter[d_id], 8))

        # build x
        x = [0]  # 0 is the index of the bias term
        for key in row:
            value = row[key]
            # one-hot encode everything with hash trick
            index = abs(hash(key + '_' + value)) % D
            x.append(index)
        yield t, ID, x, y



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'shap'
__author__ = 'JieYuan'
__mtime__ = '2019-05-09'
"""
# import xgboost
# import shap
#
# # load JS visualization code to notebook
# shap.initjs()
#
# # train XGBoost model
# X, y = shap.datasets.boston()
# model = xgboost.train({"learning_rate": 0.01}, xgboost.DMatrix(X, label=y), 100)
#
# # explain the model's predictions using SHAP values
# # (same syntax works for LightGBM, CatBoost, and scikit-learn models)
# explainer = shap.TreeExplainer(model)
#
# _X = X.iloc[0, :]
# shap_values = explainer.shap_values(_X)
# # visualize the first prediction's explanation (use matplotlib=True to avoid Javascript)
# shap.force_plot(explainer.expected_value, shap_values, _X)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = '__init__.py'
__author__ = 'JieYuan'
__mtime__ = '2019/4/2'
"""

import jovian


def commit(notebook_id=None, nb_filename=None):
    """
    sh： jovian clone notebook_id
    :param nb_filename:
    :param notebook_id: 内容覆盖566f95b138a9465aa8d17e0f1836570a -> https://jvn.io/Jie-Yuan/566f95b138a9465aa8d17e0f1836570a
    :return:
    """
    print(
        "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJmcmVzaCI6ZmFsc2UsImlkZW50aXR5Ijp7InVzZXJuYW1lIjoiSmllLVl1YW4iLCJpZCI6Njd9LCJ0eXBlIjoiYWNjZXNzIiwiZXhwIjoxNTUyOTY1NTkzLCJpYXQiOjE1NTIzNjA3OTMsIm5iZiI6MTU1MjM2MDc5MywianRpIjoiNjM1ZTg2MjQtYjA1ZC00NGJmLTljYjAtOGVjOGRmM2ExNmJkIn0.5jglhEGGs12ITl-DWWaFL-BVPhCzaDEeMKIJvEI-bbA")
    print('\n')
    jovian.commit(nb_filename=nb_filename, env_type='pip', notebook_id=notebook_id)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : pandas_profiling
# @Time         : 2019-07-08 10:56
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


import numpy as np
import pandas as pd
import pandas_profiling

df = pd.DataFrame(
    np.random.rand(100, 5),
    columns=['a', 'b', 'c', 'd', 'e']
)



profile = df.profile_report(title='EAD')
profile.to_file(output_file="output.html")


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : Text2Sequence
# @Time         : 2019-06-23 16:08
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

from sklearn.base import BaseEstimator, TransformerMixin
from tensorflow.python.keras.preprocessing.sequence import pad_sequences
from ...pipe import tqdm


class Text2Sequence(BaseEstimator, TransformerMixin):

    def __init__(self, num_words=None, maxlen=128, tokenizer=str.split):
        """OOV索引为1: embedding多了两个索引0（padding）和 1（oov）

        :param num_words:
        :param maxlen:
        :param tokenizer:
        """
        self._num_words = num_words
        self._maxlen = maxlen
        self._tokenizer = tokenizer  # 必须返回list
        self.word2index = None
        self.index2word = None

    def fit(self, X):
        if self._num_words:
            _ = {}
            for doc in tqdm(X):
                for word in self._tokenizer(doc):
                    _[word] = _.get(word, 0) + 1
            _ = dict(sorted(_.items(), key=lambda x: x[1], reverse=True)[:self._num_words])

        else:
            _ = set()
            for doc in tqdm(X):
                _.update(self._tokenizer(doc))
            print('num_words: %s' % len(_))

        self.index2word = dict(enumerate(_, 2))
        self.index2word[1] = '__OOV__'
        self.word2index = {v: k for k, v in self.index2word.items()}
        return self

    def transform(self, X, padding='post', truncating='post'):
        """

        :param X: ["我是第一条文本",]
        :param padding: 前补后补
        :param truncating: 前移除后移除
        :return:
        """
        docs = map(self._tokenizer, X)
        mapper = lambda doc: [self.word2index.get(w, 1) for w in doc]
        pad_docs = pad_sequences(list(map(mapper, docs)),
                                 self._maxlen,
                                 padding=padding,
                                 truncating=truncating)
        return pad_docs



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : lgb
# @Time         : 2019-09-16 10:06
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

from tql.algo_ml.cv import LGBMClassifierCV
from tql.algo_ml.opt_params import Optimizer
from sklearn.datasets import make_classification

X, y = make_classification(10000)


class Opt(Optimizer):

    def objective(self, **params):
        """重写目标函数"""
        self.params = params
        # 超参
        params['num_leaves'] = int(params['num_leaves'])
        params['min_child_samples'] = int(params['min_child_samples'])

        # 固定参数：TODO 更方便的方式
        params['n_estimators'] = 3000
        params['subsample_freq'] = 6  # 需要调参不
        params['verbosity'] = -1
        params['n_jobs'] = 16

        # self.params = params
        self.clf = LGBMClassifierCV(params)
        return self.clf.fit(X, y, X[:1])


opt = Optimizer(X, y)
print(opt.maximize(1))



#!/usr/bin/env python
#-*- coding: utf-8 -*-
"""
__title__ = 'get_http_list'
__author__ = 'JieYuan'
__mtime__ = '19-3-12'
"""

def get_http_list(url="http://10.114.38.22:9999"):
    import requests
    from bs4 import BeautifulSoup
    soup=BeautifulSoup(requests.get(url).text, "lxml") # soup.prettify()
    # soup.find_all('div', class_='cangtoushi-item')
    print(soup.text)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'multi_read_csv'
__author__ = 'JieYuan'
__mtime__ = '19-3-15'
"""

import pandas as pd
from concurrent.futures import ProcessPoolExecutor

from tqdm import tqdm
from functools import partial


def multi_read_csv(files, workers=2, **kwargs):
    read_func = partial(pd.read_csv, **kwargs)
    return list(map(read_func, tqdm(files)))



#!/usr/bin/env python
#-*- coding: utf-8 -*-
"""
__title__ = 'tpot'
__author__ = 'JieYuan'
__mtime__ = '19-1-3'
"""

from tpot import TPOTClassifier

TPOTClassifier(
    generations=100,
    population_size=100,
    offspring_size=None,
    mutation_rate=0.9,
    crossover_rate=0.1,
    scoring=None,
    cv=5,
    subsample=1.0,
    n_jobs=1,
    max_time_mins=None,
    max_eval_time_mins=5,
    random_state=None,
    config_dict=None,
    warm_start=False,
    memory=None,
    use_dask=False,
    periodic_checkpoint_folder=None,
    early_stop=None,
    verbosity=0,
    disable_update_check=False)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : KerasOOF
# @Time         : 2019-07-01 22:44
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  :


import time
import numpy as np
import pandas as pd
from sklearn.model_selection import StratifiedKFold, KFold
from sklearn.metrics import roc_auc_score


class KerasCV(object):
    """cross_val_predict"""

    def __init__(self, create_estimator, batch_size=128, epochs=10, callbacks=None, cv=5, random_state=None):
        self.estimators = [create_estimator() for _ in range(cv)]
        self.batch_size = batch_size
        self.epochs = epochs
        self.callbacks = callbacks
        self._kf = StratifiedKFold(cv, True, random_state)
        self._num_preds = cv

    def fit(self, X, y, X_test, feval=roc_auc_score):
        """全数组
        :param X:
        :param y:
        :param X_test:
        :param feval:
        :return:
        """
        self.oof_train = np.zeros(len(X))
        self.oof_test = np.zeros((len(X_test), self._num_preds))
        for (n_fold, (train_index, valid_index)), estimator in zip(enumerate(self._kf.split(X, y)), self.estimators):
            print(f"\033[94mFold {n_fold + 1} started at {time.ctime()}\033[0m")
            X_train, y_train = X[train_index], y[train_index]
            X_valid, y_valid = X[valid_index], y[valid_index]
            # eval_set = [(X_train, y_train), (X_valid, y_valid)]
            estimator.fit(X_train, y_train,
                          validation_data=(X_valid, y_valid),
                          batch_size=self.batch_size,
                          epochs=self.epochs,
                          callbacks=self.callbacks)

            ########################################################################
            # 二分类
            self.oof_train[valid_index] = estimator.predict(X_valid)[:, -1]  # num_sample * 1
            self.oof_test[:, n_fold] = estimator.predict(X_test)[:, -1]
            ########################################################################

        # 输出 测试集 oof
        self.oof_test_rank = pd.DataFrame(self.oof_test).rank().mean(1) / len(self.oof_test)
        self.oof_test = self.oof_test.mean(1)

        # 计算 训练集 oof 得分
        if feval:
            score = feval(y, self.oof_train)
            print(f"\n\033[94mCV Sorce: {score} ended at {time.ctime()}\033[0m")
            return score

    def oof_save(self, file='./oof_train_and_test.csv'):
        assert isinstance(file, str)
        _ = np.append(self.oof_train, self.oof_test)
        pd.DataFrame(_, columns='oof_train_and_test').to_csv(file, index=False)


if __name__ == '__main__':
    from tensorflow.python.keras.layers import Dense
    from tensorflow.python.keras.models import Sequential


    def create_model():
        """from tensorflow.python.keras.models import clone_and_build_model"""
        model = Sequential()
        model.add(Dense(12, input_dim=20, kernel_initializer="uniform", activation="relu"))
        model.add(Dense(8, kernel_initializer="uniform", activation="relu"))
        model.add(Dense(1, kernel_initializer="uniform", activation="sigmoid"))
        model.compile(loss="binary_crossentropy", optimizer="adam", metrics=["accuracy"])
        return model


    oof = KerasCV(create_estimator=create_model)
    from sklearn.datasets import make_classification

    X, y = make_classification(10000, shift=0.1)
    oof.fit(X, y, X)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : report
# @Time         : 2019-07-10 13:38
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


import numpy as np
import pandas as pd
import pandas_profiling

# df = pd.DataFrame(
#     np.random.rand(100, 5),
#     columns=['a', 'b', 'c', 'd', 'e']
# )

def get_html_str(**kwargs):
    df_dic = kwargs['df_dic']
    df = pd.DataFrame(df_dic)
    return df.profile_report().to_html()


from restful_api import Api
api = Api('/post1', get_html_str, verbose=False)
api.app.run('0.0.0.0')


# import numpy as np
# import pandas as pd
# import pandas_profiling
#
# df = pd.DataFrame(
#     np.random.rand(100, 5),
#     columns=['a', 'b', 'c', 'd', 'e']
# )
#
# import requests
#
# url = 'http://0.0.0.0:8000/post1'
# json = {'df_dic': df.to_dict()}
# r = requests.post(url, json=json)
# html_str = r.json()['Score']


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'HotSpider'
__author__ = 'JieYuan'
__mtime__ = '2019-05-15'
"""
from ..pipe import tqdm

import requests
import pandas as pd
from lxml.etree import HTML
from fake_useragent import UserAgent


class HotSpider(object):

    def __init__(self, query='news'):
        """

        :param query:
        :param categories:
        """
        self.query = query
        self.categories = ['news', 'tech', 'ent', 'shopping', 'community']
        self.ua = UserAgent()

    def df_sites_info(self):
        self.urls = self.df_sites.url

        dfs = []
        for url in tqdm(self.urls):
            r = self._request(url)
            dom_tree = HTML(r.text)
            site = dom_tree.xpath('normalize-space(//div[@class="Xc-ec-L b-L"]/text())')
            print('🕷：%s %s' % (site, url))

            df = pd.read_html(self._request(url).text)[0]
            df.columns = ['rank', 'title', 'hot', 'site']
            df['site'] = site
            df['url'] = url
            dfs.append(df)
        return pd.concat(dfs)  # pd.merge(self.df_sites, pd.concat(dfs))

    @property
    def df_sites(self):
        print('🕷：%s ...' % self.query)
        if self.query in self.categories:
            url = 'https://tophub.today/c/' + self.query
            r = self._request(url)
            dom_tree = HTML(r.text)
            pages = int(dom_tree.xpath('//div/small/text()')[0][:-1]) // 12 + 1
            urls = [url + '?p=%s' % i for i in range(1, pages + 1)]
            print('%s: %s' % (self.query, pages))
        else:
            urls = ['https://tophub.today/search?q=' + self.query]

        dfs = []
        for url in urls:
            r = self._request(url)
            dom_tree = HTML(r.text)
            urls = dom_tree.xpath('//div/a[starts-with(@href,"/n")]/@href')
            subscriber_num = dom_tree.xpath('//span[@class="zb-Rb-subscriber-Wc"]/text()')

            df_sites = pd.DataFrame()
            df_sites['url'] = ['https://tophub.today' + i for i in urls]
            df_sites['subscriber_num'] = pd.Series(subscriber_num).astype(int)
            df_sites.sort_values('subscriber_num', 0, False, True)
            dfs.append(df_sites)
        return pd.concat(dfs)

    @property
    def headers(self):
        return {'user-agent': self.ua.random}

    def _request(self, url):
        try:
            r = requests.get(url, timeout=500, headers=self.headers)
            r.raise_for_status()
            r.encoding = 'utf8'  # r.apparent_encoding
            return r

        except Exception as e:
            print('🕷：%s' % e)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'utils'
__author__ = 'JieYuan'
__mtime__ = '2019-05-15'
"""
# from fake_useragent import UserAgent


class SpiderConfig(object):
    # ua = UserAgent()

    def __init__(self):
        pass

    @property
    def headers(self):
        return {'user-agent': self.ua.random}

    @property
    def proxies(self):
        pass

    @property
    def cookies(self):
        pass


# import random
# import requests
#
# ips = [("118.190.95.35", 9001), ]
#
# try:
#     requests.adapters.DEFAULT_RETRIES = 3
#     host, port = random.choice(ips)
#     proxies = {"http": "http://%s:%s" % (host, port)}
#
#     r = requests.get("http://icanhazip.com/", timeout=8, proxies=proxies)
#     if host == r.text.strip():
#         print('代理有效：%s' % proxies )
#     else:
#         print("代理无效")
# except:
#     print("代理无效")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : FeatureColumnsAndFunctionalAPI
# @Time         : 2019-07-12 17:46
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

"""
https://github.com/tensorflow/tensorflow/issues/27416
"""
from __future__ import absolute_import, division, print_function

import numpy as np
import pandas as pd

#!pip install tensorflow==2.0.0-alpha0
import tensorflow as tf

from tensorflow import feature_column
from tensorflow import keras
from tensorflow.python.keras import layers
from sklearn.model_selection import train_test_split

URL = 'https://storage.googleapis.com/applied-dl/heart.csv'
dataframe = pd.read_csv(URL)
dataframe.head()

train, test = train_test_split(dataframe, test_size=0.2)
train, val = train_test_split(train, test_size=0.2)
print(len(train), 'train examples')
print(len(val), 'validation examples')
print(len(test), 'test examples')

# A utility method to create a tf.data dataset from a Pandas Dataframe
def df_to_dataset(dataframe, shuffle=True, batch_size=32):
  dataframe = dataframe.copy()
  labels = dataframe.pop('target')
  ds = tf.data.Dataset.from_tensor_slices((dict(dataframe), labels))
  if shuffle:
    ds = ds.shuffle(buffer_size=len(dataframe))
  ds = ds.batch(batch_size)
  return ds

batch_size = 5 # A small batch sized is used for demonstration purposes
train_ds = df_to_dataset(train, batch_size=batch_size)
val_ds = df_to_dataset(val, shuffle=False, batch_size=batch_size)
test_ds = df_to_dataset(test, shuffle=False, batch_size=batch_size)

age = feature_column.numeric_column("age")

feature_columns = []
feature_layer_inputs = {}

# numeric cols
for header in ['age', 'trestbps', 'chol', 'thalach', 'oldpeak', 'slope', 'ca']:
  feature_columns.append(feature_column.numeric_column(header))
  feature_layer_inputs[header] = tf.keras.Input(shape=(1,), name=header)

# bucketized cols
age_buckets = feature_column.bucketized_column(age, boundaries=[18, 25, 30, 35, 40, 45, 50, 55, 60, 65])
feature_columns.append(age_buckets)

# indicator cols
thal = feature_column.categorical_column_with_vocabulary_list(
      'thal', ['fixed', 'normal', 'reversible'])
thal_one_hot = feature_column.indicator_column(thal)
feature_columns.append(thal_one_hot)
feature_layer_inputs['thal'] = tf.keras.Input(shape=(1,), name='thal', dtype=tf.string)

# embedding cols
thal_embedding = feature_column.embedding_column(thal, dimension=8)
feature_columns.append(thal_embedding)

# crossed cols
crossed_feature = feature_column.crossed_column([age_buckets, thal], hash_bucket_size=1000)
crossed_feature = feature_column.indicator_column(crossed_feature)
feature_columns.append(crossed_feature)

batch_size = 32
train_ds = df_to_dataset(train, batch_size=batch_size)
val_ds = df_to_dataset(val, shuffle=False, batch_size=batch_size)
test_ds = df_to_dataset(test, shuffle=False, batch_size=batch_size)

feature_layer = tf.keras.layers.DenseFeatures(feature_columns)
feature_layer_outputs = feature_layer(feature_layer_inputs)

x = layers.Dense(128, activation='relu')(feature_layer_outputs)
x = layers.Dense(64, activation='relu')(x)

baggage_pred = layers.Dense(1, activation='sigmoid')(x)

model = keras.Model(inputs=[v for v in feature_layer_inputs.values()], outputs=baggage_pred)

model.compile(optimizer='adam',
              loss='binary_crossentropy',
              metrics=['accuracy'])

model.fit(train_ds, epochs=10)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : EmbeddingFastText
# @Time         : 2019-06-23 17:28
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 
import numpy as np
from sklearn.base import BaseEstimator, TransformerMixin
from tensorflow.python.keras.preprocessing.sequence import pad_sequences
from ...pipe import tqdm


class Text2SequenceByFastText(BaseEstimator, TransformerMixin):
    """
    word2id['<PAD>'] = 0
    word2id['<START>'] = 1
    word2id['<UNK>'] = 2
    word2id['<UNUSED>'] = 3
    """

    def __init__(self, maxlen=128, fasttext_model=None, tokenizer=str.split):
        """

        :param maxlen:
        :param fasttext_model: from gensim.models.fasttext import load_facebook_model
        :param tokenizer: 必须返回列表
        """
        self.maxlen = maxlen
        self.model = fasttext_model
        self._tokenizer = tokenizer
        self.word2index = None
        self.weights = None

    def fit(self, X):
        """数据转换的过程中得到embedding的weights
        :param X:
        :return:
        """
        index2word = dict(enumerate(['__PAD__', '__OOV__'] + self.model.wv.index2entity))
        self.word2index = {v: k for k, v in index2word.items()}

        self.weights = self.model.wv.vectors
        rand_vecs = np.random.normal(size=(2, self.weights.shape[1]))  # padding/oov
        self.weights = np.row_stack((rand_vecs, self.weights))

        _array = []
        for doc in tqdm(X):
            for w in self._tokenizer(doc):
                if w not in self.word2index:
                    self.word2index[w] = len(self.word2index)
                    _array.append(self.model.wv[w])  # TODO: 非fasttext怎么解决OOV? 直接补位1（OOV标识）
        print("New Words: %s" % len(_array))

        self.weights = np.row_stack((self.weights, *_array))
        return self

    def transform(self, X, padding='post', truncating='post'):
        """

        :param X:
        :param padding: 前补后补
        :param truncating: 前移除后移除
        :return:
        """
        docs = map(self._tokenizer, X)
        mapper = lambda doc: [self.word2index.get(w, 1) for w in doc]  # 1: __OOV__
        pad_docs = pad_sequences(list(map(mapper, docs)),
                                 self.maxlen,
                                 padding=padding,
                                 truncating=truncating)
        return pad_docs



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : server
# @Time         : 2019-07-30 18:56
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


import pyspark
from pyspark.sql.types import *

import pyspark.sql.functions as F
from pyspark.ml.feature import Word2Vec

spark = pyspark.sql.SparkSession.builder.appName("MyApp") \
    .config("spark.jars.packages", "Azure:mmlspark:0.17") \
    .getOrCreate()

from sklearn.datasets import make_classification
import pandas as pd

X, y = make_classification(n_features=4)
df_pandas = pd.DataFrame(X).assign(label=y)
df_pandas.columns = [str(i) for i in df_pandas.columns]
df = spark.createDataFrame(df_pandas)

from pyspark.ml.classification import LogisticRegression
from mmlspark import LightGBMClassifier, TrainClassifier, ComputeModelStatistics

model = TrainClassifier(model=LogisticRegression(), labelCol='label').fit(df)

from pyspark.sql.types import *
import uuid

serving_inputs = spark.readStream.server() \
    .address("0.0.0.0", 8899, "my_api") \
    .option("name", "my_api") \
    .load() \
    .parseRequest("my_api", df.schema)

serving_outputs = model.transform(serving_inputs) \
    .makeReply("scored_labels")

server = serving_outputs.writeStream \
    .server() \
    .replyTo("my_api") \
    .queryName("my_query") \
    .option("checkpointLocation", "file:///tmp/checkpoints-{}".format(uuid.uuid1())) \
    .start()



#!/usr/bin/env python
#-*- coding: utf-8 -*-
"""
__title__ = 'proxies'
__author__ = 'JieYuan'
__mtime__ = '2019-05-15'
"""
import requests
import pandas as pd
from tqdm import tqdm


class Proxies(object):

    def __init__(self, ):
        url = 'https://www.xicidaili.com/nn/'
        r = requests.get(url, headers={'user-agent': 'Mozilla/5.0'})
        self.ips = pd.read_html(r.text, header=0)[0][['IP地址', '端口']].values

    @property
    def pool(self):
        proxies = []
        for i in tqdm(self.ips):
            proxy = self._get_valid_proxies(i)
            if proxy:
                proxies.append(proxy)
        return proxies

    def _get_valid_proxies(self, ip):
        try:
            requests.adapters.DEFAULT_RETRIES = 3
            host, port = ip  # random.choice(ips)
            proxies = {"http": "http://%s:%s" % (host, port)}
            r = requests.get("http://icanhazip.com/", timeout=8, proxies=proxies)
            if host == r.text.strip():
                # print('代理有效：%s' % proxies)
                return proxies
            else:
                print("pass")  # 代理无效
        except:
            print("pass")


if __name__ == '__main__':
    pool = Proxies().pool
    print(pool)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'topnews'
__author__ = 'JieYuan'
__mtime__ = '2019-05-07'
"""
import re
import requests
import pandas as pd
from bs4 import BeautifulSoup


class HotNews(object):

    def __init__(self):
        pass

    def tuling(self, q):
        url = 'http://www.tuling123.com/openapi/api?key=3ac26126997942458c0d93de30d52212&info='
        return requests.get(url + q).json()['text']

    def sina(self):
        url = 'http://www.sina.com.cn/mid/search-list.shtml'
        soup = self._get_soup(url)
        soups = soup.findAll('div', re.compile('keyword|exp'))

        news = []
        for idx in range(0, len(soups), 2):
            s1 = soups[idx].text.split(r'#')[1]
            s2 = soups[idx + 1].text
            news.append([s1, s2])

        return news

    def weibo(self):
        url = 'https://s.weibo.com/top/summary?cate=realtimehot'
        # df = pd.read_html(url)[0][['关键词']]
        soup = self._get_soup(url)
        soups = soup.findAll(re.compile('td|span'), 'td-02')

        news = []
        for s in soups:
            s1 = s.a.text
            s2 = s.span.text if s.span else ''
            news.append([s1, s2])
        return news

    def baidu(self, period=0):
        _map = {0: 1, 1: 341, 7: 42}
        url = 'http://top.baidu.com/buzz?b=%s' % _map.get(period, _map[0])

        df = pd.read_html(url, header=0)[0].dropna().drop(['排名', '相关链接'], 1)
        return df

    def sougou(self, period=0):
        _map = {0: 'shishi', 7: 'sevendsnews'}

        news = []
        for i in range(3):
            url = 'http://top.sogou.com/hot/%s_%s.html' % (_map.get(period, _map[0]), i + 1)
            soup = self._get_soup(url)

            soups = soup.findAll(re.compile('span|p'), re.compile('s3|p1|p3'))
            for idx in range(0, len(soups), 2):
                s1 = soups[idx].text
                s2 = soups[idx + 1].text
                s3 = soups[idx + 1].find().attrs['class'][0]
                news.append([s1, s2, s3])
        return news

    def qihu(self, period=0):
        _map = {0: 'https://news.so.com/hotnews',
                1: 'https://trends.so.com/top/realtime'}
        url = _map.get(period, _map[1])

        if period:
            r = requests.get(url, timeout=300, headers={'user-agent': 'Mozilla/5.0'})
            r.encoding = r.apparent_encoding
            return pd.DataFrame(r.json()['data']['result'])
        else:
            soup = self._get_soup(url)

            news = []
            soups = soup.findAll('span', class_=re.compile('title|hot'))
            for idx in range(0, len(soups), 2):
                s1 = soups[idx].string
                s2 = soups[idx + 1].string
                news.append([s1, s2])
            return news

    def _get_soup(self, url='https://www.baidu.com'):
        try:
            r = requests.get(url, timeout=300, headers={'user-agent': 'Mozilla/5.0'})
            r.raise_for_status()
            r.encoding = r.apparent_encoding
            return BeautifulSoup(r.text)
        except:
            print('爬取失败')



if __name__ == '__main__':
    hn = HotNews()
    print(pd.DataFrame(hn.sina()))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'limit_memory'
__author__ = 'JieYuan'
__mtime__ = '19-3-15'
"""
import resource


def limit_memory(memory=16):
    """
    :param memory: 默认限制内存为 16G
    :return:
    """
    rsrc = resource.RLIMIT_AS
    # res_mem=os.environ["RESOURCE_MEM"]
    memlimit = memory * 1024 ** 3
    resource.setrlimit(rsrc, (memlimit, memlimit))
    # soft, hard = resource.getrlimit(rsrc)
    print("memory limit as: %s G" % memory)


#!/usr/bin/env python
#-*- coding: utf-8 -*-
"""
__title__ = 'autokeras'
__author__ = 'JieYuan'
__mtime__ = '19-1-3'
"""

import autokeras as ak

# ak.MlpModule
#
# ak.ImageClassifier
# ak.ImageRegressor
#
# ak.TabularClassifier
# ak.TabularRegressor
# ak.TextClassifier
# ak.TextRegressor


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : sfs
# @Time         : 2019-07-26 13:41
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  :

import joblib
from sklearn.feature_selection import RFECV
from mlxtend.feature_selection import ExhaustiveFeatureSelector as EFS
from mlxtend.feature_selection import SequentialFeatureSelector as SFS
from mlxtend.plotting import plot_sequential_feature_selection as plot_sfs
from sklearn.neighbors import NearestNeighbors
from sklearn.feature_selection import GenericUnivariateSelect, \
    SelectPercentile, SelectKBest, f_classif, mutual_info_classif, RFE

from lightgbm import LGBMClassifier
import matplotlib.pyplot as plt


class FeatureSelector(object):

    def __init__(self, estimator=LGBMClassifier(n_jobs=-1), scoring='roc_auc', selector_name='rfe', cv_worker=5):
        """

        :param estimator:
        :param scoring:
        :param selector: {'sfs', 'efs', 'rfe'} # rfe 可以粗排一下用gpu加速
        :param cv_worker: gpu要设为1
        """
        self.selector_name = selector_name
        if selector_name == 'sfs':
            """
            efs的优化版：根据 scoring 筛选特征
            顺序特征选择算法的贪婪搜索算法家族，用于减少初始d维特征空间到ķ维特征空间，其中ķ<d 。
            特征选择算法背后的动机是自动选择与问题最相关的特征子集。
            特征选择的目标是双重的：我们希望通过去除不相关的特征或噪声来提高计算效率并减少模型的泛化误差。
            如果嵌入式特征选择（例如，像LASSO这样的正则化惩罚）不适用，则诸如顺序特征选择之类的包装器方法尤其有用。
            """
            self.selector = SFS(estimator,
                                scoring=scoring,
                                cv=5,
                                n_jobs=cv_worker,
                                verbose=2,

                                k_features='best',
                                forward=True,
                                floating=False,
                                )
        elif selector_name == 'efs':  # 枚举：2^n
            self.selector = EFS(estimator,
                                scoring=scoring,
                                cv=5,
                                n_jobs=cv_worker,
                                print_progress=True,

                                max_features=1000
                                )
        elif selector_name == 'rfe':  # 根据树模型特征重要性等权重信息筛选特征
            """https://www.jianshu.com/p/025395835591
            # 打印的是相应位置上属性的排名
            print(rfe.ranking_)
            # 属性选择的一种模糊表示，选择的是true，未选择的是false
            print(rfe.support_)
            # 第1个属相的排名
            print(rfe.ranking_[1])
            # 外部估计函数的相关信息
            print(rfe.estimator_)
            """
            """https://www.kaggle.com/roydatascience/recursive-feature-selection-new-transactions-elo"""
            self.selector = RFECV(
                estimator,
                scoring=scoring,
                cv=5,
                n_jobs=cv_worker,
                verbose=2,

                step=1 / 64,  # 每次迭代要删除的特征数/占比
            )

    def fit(self, X, y, model_name=None):
        self.dim = X.shap[1]
        self.selector.fit(X, y)  # 可重写 fit
        if model_name:
            joblib.dump(self.selector, model_name)

    def plot(self):
        if self.selector_name == 'rfe':
            step = self.selector.sep if self.selector.sep > 1 else int(self.selector.sep * self.dim)
            plt.figure(figsize=(12, 9))
            plt.xlabel(f'Number of features tested x {step}')
            plt.ylabel('Cross-validation score')
            plt.plot(range(1, len(self.selector.grid_scores_) + 1), self.selector.grid_scores_)
            # plt.savefig('ELO-lgbmcv-02.png', dpi=150)
            plt.show()

        plot_sfs(self.selector.get_metric_dict(), kind='std_dev')
        plt.ylim([0.8, 1])
        plt.title('Sequential Forward Selection (w. StdDev)')
        plt.show()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : word_count
# @Time         : 2019-08-08 10:10
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


from tql.spark import SparkInit

sc, spark = SparkInit()()
tf = (sc.textFile('./corpus.txt')
 .flatMap(lambda line: line.split(" "))
 .map(lambda word: (word, 1))
 .reduceByKey(lambda a, b: a + b)
 .collect())


print(sorted(tf, key=lambda x: - x[1]))


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : knn
# @Time         : 2019-07-26 14:24
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  :

"""
https://www.jianshu.com/p/778d39132d66
"""
import numpy as np

from sklearn.neighbors import NearestNeighbors, KNeighborsClassifier, RadiusNeighborsClassifier

# 归一化
X_train = ...
X_test = ...

neigh = NearestNeighbors(5, n_jobs=-1)
neigh.fit(X_train)

dists, _ = neigh.kneighbors(X_train, n_neighbors=5)
mean_dist = dists.mean(axis=1)
max_dist = dists.max(axis=1)
min_dist = dists.min(axis=1)

X_train['300'] = X_train.std(1)
X_train = np.hstack((X_train, mean_dist.reshape(-1, 1), max_dist.reshape(-1, 1), min_dist.reshape(-1, 1)))

test_dists, _ = neigh.kneighbors(X_test, n_neighbors=5)

test_mean_dist = test_dists.mean(axis=1)
test_max_dist = test_dists.max(axis=1)
test_min_dist = test_dists.min(axis=1)

X_test['300'] = X_test.std(1)
X_test = np.hstack((X_test, test_mean_dist.reshape(-1, 1), test_max_dist.reshape(-1, 1), test_min_dist.reshape(-1, 1)))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : bert_keras
# @Time         : 2019-06-21 14:43
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


# import pandas as pd
# from sklearn.model_selection import train_test_split
# from nn.keras.utils import _DataGenerator
# from keras_bert import load_trained_model_from_checkpoint, Tokenizer, load_vocabulary
#
# # bert_dir = '/Users/yuanjie/Desktop/Data/chinese_L-12_H-768_A-12'
# # data_path = './sentiment.tsv.zip'
#
# bert_dir = '/fds/data/wv/chinese_L-12_H-768_A-12'
# data_path = '/fds/data/sentiment.tsv.zip'
#
# config_path = bert_dir + '/bert_config.json'
# checkpoint_path = bert_dir + '/bert_model.ckpt'
# dict_path = bert_dir + '/vocab.txt'
# token_dict = load_vocabulary(dict_path)
# bert_model = load_trained_model_from_checkpoint(config_path, checkpoint_path)
#
# for l in bert_model.layers:
#     l.trainable = True
#
#
# # 重写Token
# class OurTokenizer(Tokenizer):
#     def _tokenize(self, text):
#         R = []
#         for c in text:
#             if c in self._token_dict:
#                 R.append(c)
#             elif self._is_space(c):
#                 R.append('[unused1]')  # space类用未经训练的[unused1]表示
#             else:
#                 R.append('[UNK]')  # 剩余的字符是[UNK]
#         return R
#
#
# tokenizer = OurTokenizer(token_dict)
#
# from keras.layers import *
# from keras.models import Model
# from keras.optimizers import Adam
#
# x1_in = Input(shape=(None,))
# x2_in = Input(shape=(None,))
#
# x = bert_model([x1_in, x2_in])
# x = Lambda(lambda x: x[:, 0])(x)
# p = Dense(1, activation='sigmoid')(x)
#
# model = Model([x1_in, x2_in], p)
# model.compile(
#     loss='binary_crossentropy',
#     optimizer=Adam(1e-5),  # 用足够小的学习率
#     metrics=['accuracy']
# )
# model.summary()
#
# ######################################################
# from keras.preprocessing.sequence import pad_sequences
#
#
# def mapper(X, y):
#     X = list(map(lambda x: pad_sequences(x, 256), zip(*map(tokenizer.encode, X))))
#     return X, y
#
#
# ######################################################
#
# df = pd.read_csv(data_path, '\t')
# X = df.text.astype(str)
# y = df.label.values.reshape(-1, 1)
#
# X_train, X_test, y_train, y_test = train_test_split(
#     X, y, test_size=0.25, random_state=42)
#
# dg_train = _DataGenerator(X_train, y_train, 32, mapper)
# dg_valid = _DataGenerator(X_test, y_test, 32, mapper)
# model.fit_generator(dg_train.__iter__(), len(dg_train),
#                     epochs=10,
#                     validation_data=dg_valid.__iter__(),
#                     validation_steps=len(dg_valid))


# encoding: utf-8

from .TextClean import TextClean
from .Sent2Vec import Sent2Vec
from .Sequence import Sequence
from .Text2Sequence import Text2Sequence
from .Text2SequenceByFastText import Text2SequenceByFastText
from .WordVec2BOW import WordVec2BOW
from .Emoji import Emoji
from .ngrams import ngrams, everygrams, skipgrams



# -*- coding: utf-8 -*-
"""
__title__ = 'time_utils'
__author__ = 'JieYuan'
__mtime__ = '2018/7/27'
"""
import time
import numpy as np

# pd.datetime.now().timestamp() # pd.datetime.now(cst_tz).timestamp()# 时间戳相差8*3600
# pd.read_csv(parse_dates)


# 时间戳 转 时间字符串
def timestamp2str(timestamp, format=''):
    """
    t = pd.datetime.now().timestamp()
    ts = pd.Series([t]*10, name='t')

    # 时间戳 转 时间字符串
    ts = ts.map(timestamp2str) # 会有时区问题 %Y-%m-%d %H:%M:%S
    # 时间字符串 转 时间
    ts = ts.astype('datetime64[ns]') # 慢一些 pd.to_datetime(ts, errors='coerce', infer_datetime_format=True)

    # 时间 转 时间戳
    ts.map(lambda x: x.timestamp())

    """
    return time.strftime(format, time.localtime(timestamp))


def exponential_decay(t, delta=24 * 7, start=1, end=0):
    """
    "拟合随时间指数衰减的过程"
    https://blog.csdn.net/xiaokang06/article/details/78076925
    https://blog.csdn.net/zhufenghao/article/details/80879260

    """
    alpha = np.log(start / (end + 1e-8)) / delta
    t0 = - np.log(start) / alpha

    decay = np.exp(-alpha * (t + t0))
    return decay



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : mid_layer
# @Time         : 2019-07-12 16:33
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 
"""
https://www.tensorflow.org/beta/tutorials/keras/feature_columns
"""

from tensorflow.python.keras.layers import Input, Embedding, Reshape, Activation
from tensorflow.python.keras.models import Model

input_model = Input(shape=(1,))
output_store = Embedding(1115, 10, name='store_embedding')(input_model)
output_store = Reshape(target_shape=(10,))(output_store)

output_model = Activation('sigmoid')(output_store)
model = Model(inputs=input_model, outputs=output_model)
model.summary()

embed = Model(inputs=model.input, outputs=model.get_layer(index=1).output)
# 以这个model的预测值作为输出
embed.predict([[1]])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : TextRCNN
# @Time         : 2019-06-24 17:15
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

from .BaseModel import BaseModel
from tensorflow.python.keras import backend as K
from tensorflow.python.keras import Input, Model
from tensorflow.python.keras.layers import Dense, Embedding, Lambda, Concatenate
from tensorflow.python.keras.layers import Conv1D, GlobalMaxPooling1D
from tensorflow.python.keras.layers import Bidirectional, LSTM, CuDNNLSTM, GRU, CuDNNGRU


class TextRCNN(BaseModel):
    def __init__(self, max_tokens, maxlen=128, embedding_size=None, num_class=1, weights=None, rnn=CuDNNGRU):
        """
        :param rnn: LSTM, CuDNNLSTM, GRU, CuDNNGRU
        """
        super().__init__(max_tokens, maxlen, embedding_size, num_class, weights)
        self._RNN = rnn

    def get_model(self):
        input_current = Input((self.maxlen,))
        input_left = Input((self.maxlen,))
        input_right = Input((self.maxlen,))

        embedder = Embedding(self.max_tokens, self.embedding_size, input_length=self.maxlen)
        embedding_current = embedder(input_current)
        embedding_left = embedder(input_left)
        embedding_right = embedder(input_right)

        x_left = self._RNN(128, return_sequences=True)(embedding_left)
        x_right = self._RNN(128, return_sequences=True, go_backwards=True)(embedding_right)

        x_right = Lambda(lambda x: K.reverse(x, axes=1))(x_right)
        x = Concatenate(axis=2)([x_left, embedding_current, x_right])

        x = Conv1D(64, kernel_size=1, activation='tanh')(x)
        x = GlobalMaxPooling1D()(x)

        output = Dense(self.num_class, activation=self.last_activation)(x)
        model = Model(inputs=[input_current, input_left, input_right], outputs=output)
        return model



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'FeatureColumns'
__author__ = 'JieYuan'
__mtime__ = '19-3-22'
"""
import tensorflow as tf

"""
https://blog.csdn.net/u014061630/article/details/82937333
https://www.tensorflow.org/beta/tutorials/keras/feature_columns
https://github.com/tensorflow/tensorflow/issues/27416
"""

# 数值列
# 分桶列
_ = tf.feature_column.numeric_column(key='Year')
tf.feature_column.bucketized_column(source_column=_, boundaries=[1960, 1980, 2000])

# 分类标识列
tf.feature_column.categorical_column_with_identity(key='my_feature_b', num_buckets=4)  # Values [0, 4)

# 分类词汇列：分类标识列的枚举版本（多了一步词到数值映射）
tf.feature_column.categorical_column_with_vocabulary_list(key='feat1',
                                                          vocabulary_list=["kitchenware", "electronics", "sports"])
tf.feature_column.categorical_column_with_vocabulary_file(key='feat1',
                                                          vocabulary_file='./cats.txt')

# 经过哈希处理的列
# tf.feature_column.categorical_column_with_hash_bucket(key=_,
#                                                       hash_buckets_size=10)  # The number of categories
# 组合列
# https://developers.google.com/machine-learning/glossary/#feature_cross
# https://www.tensorflow.org/guide/feature_columns#crossed_column
tf.feature_column.crossed_column(['latitude_bucket_fc', 'longitude_bucket_fc'], 5000)  # using 5000 hash bins.

# 指标列和嵌入列
# tf.feature_column.indicator_column()  # one-hot
# tf.feature_column.embedding_column(categorical_column=categorical_column, dimension=embedding_dimensions)


# if __name__ == '__main__':
#     # 低阶 api
#     features = {
#         'sales': [[5], [10], [8], [9]],
#         'department': ['sports', 'sports', 'gardening', 'gardening']}
#
#     department_column = tf.feature_column.categorical_column_with_vocabulary_list(
#         'department', ['sports', 'gardening'])
#     department_column = tf.feature_column.indicator_column(department_column)
#
#     columns = [
#         tf.feature_column.numeric_column('sales'),
#         department_column
#     ]
#
#     inputs = tf.feature_column.input_layer(features, columns)
#     var_init = tf.global_variables_initializer()
#     table_init = tf.tables_initializer()
#
#     with tf.Session() as sess:
#         sess.run((var_init, table_init))
#         print(sess.run(inputs))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : is_in_notebook
# @Time         : 2019-06-20 11:16
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


def _in_notebook():
    try:
        from IPython import get_ipython

        if 'IPKernelApp' not in get_ipython().config:
            raise ImportError("Not In Notebook !")
    except:
        return False

    else:
        return True


if __name__ == '__main__':
    print(_in_notebook())



# -*- coding: utf-8 -*-
"""
__title__ = 'hive'
__author__ = 'JieYuan'
__mtime__ = '18-11-19'
"""


class Hive(object):
    get_feats = lambda feats: [i.split('.')[1] if '.' in i else i for i in feats]

    def gen_cols(self, feats_str: list = [], feats_num: list = [], all_feats: list = []):
        if all_feats:
            if feats_str or feats_num:
                if feats_str:
                    z = zip(range(1024), ['string' if feat in feats_str else 'double' for feat in all_feats], all_feats)
                else:
                    z = zip(range(1024), ['double' if feat in feats_num else 'string' for feat in all_feats], all_feats)

            res = '\n'.join([f"{idx}:optional {dtype} {feat};//{feat}" for idx, dtype, feat in z])
        else:
            z = zip(range(1024), len(feats_str) * ['string'] + len(feats_num) * ['double'], feats_str + feats_num)
            res = '\n'.join([f"{idx}:optional {dtype} {feat};//{feat}" for idx, dtype, feat in z])
        print(res)

    def gen_agg_cols(self, feats: list, funcs: list = None):
        """
        :param column: str
        :param funcs: list or tuple
            'mean', 'max', 'min', ...
            mad skew kurt待增加
        :return:
        """
        exprs = lambda column: {
            'sum': f'AVG({column}) AS {column}_sum',
            'mean': f'AVG({column}) AS {column}_mean',
            'max': f'MAX({column}) AS {column}_max',
            'min': f'MIN({column}) AS {column}_min',
            'range': f'MAX({column}) - MIN({column}) AS {column}_range',
            'std': f'STDDEV_SAMP({column}) AS {column}_std',
            'per25': f'PERCENTILE_APPROX({column}, 0.25) AS {column}_per25',
            'per50': f'PERCENTILE_APPROX({column}, 0.50) AS {column}_per50',
            'per75': f'PERCENTILE_APPROX({column}, 0.75) AS {column}_per75',
            'iqr': f'PERCENTILE_APPROX({column}, 0.75) - PERCENTILE_APPROX({column}, 0.25) AS {column}_iqr',
            'cv': f'STDDEV_SAMP({column}) / (AVG({column}) + pow(10, -8)) AS {column}_cv',
            'zeros_num': f'COUNT(CASE WHEN {column} = 0 THEN 1 ELSE NULL END) AS {column}_zeros_num',
            'zeros_perc': f'COUNT(CASE WHEN {column} = 0 THEN 1 ELSE NULL END) / COUNT(1) AS {column}_zeros_perc'
        }

        if funcs is None:
            res = ',\n'.join([',\n'.join(exprs(feat).values()) for feat in feats])
        else:
            assert isinstance(funcs, tuple) or isinstance(funcs, list)
            res = ',\n'.join([',\n'.join([exprs(feat)[func] for func in funcs]) for feat in feats])
        print(res)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : iWork.
# @File         : cli
# @Time         : 2019-06-14 12:20
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


import fire


# fire.Fire(lambda obj: type(obj).__name__)


def add(x, y, **kwargs):
    """add"""
    print(kwargs)
    return x + y


def multiply(x, y):
    """add"""
    return x * y


def cli():
    fire.Fire()


if __name__ == '__main__':
    fire.Fire()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'nms'
__author__ = 'JieYuan'
__mtime__ = '2019-05-14'
"""

import numpy as np
import nmslib
from collections import OrderedDict


class VecQuery(object):
    """
    index.getDistance
    index.knnQueryBatch
    index.loadIndex
    index.saveIndex
    """

    def __init__(self, index=None):
        """
        # TODO：index, id2word 都要存才能保证下次加载判断逻辑正确
        :param index: index.loadIndex('*')
        """
        self.id2word = {}
        self.word2id = {}
        self.index = index

    def __call__(self, *args, **kwargs):
        return self.query(*args, **kwargs)

    def query(self, data, k=10, num_threads=4):
        _ = self.index.knnQueryBatch(data, k, num_threads)
        return list(map(self._parse_result, _))

    def createIndex(self, words, vectors):
        """
        :param words: 务必唯一不重复
        :param vectors:
        :return:
        """
        if not self.id2word:
            print('Index Create ...')
            self.id2word, self.word2id = self._add(words)
            self._create(vectors)
        else:
            print('Index Add...')
            _add_ids = []
            for id, word in enumerate(words):
                if word not in self.word2id:  # 忽略已存在的索引
                    self.word2id[word] = len(self.word2id)
                    _add_ids.append(id)
            vectors = np.array(vectors)[_add_ids]
            self._create(vectors, len(self.id2word) + np.arange(len(_add_ids)))  # 增量更新索引
            self.id2word = {j: i for i, j in self.word2id.items()}

    def _add(self, words):
        id2word = dict(enumerate(words))
        word2id = {j: i for i, j in id2word.items()}
        return id2word, word2id

    def _create(self, vectors, ids=None):
        # initialize a new index, using a HNSW index on Cosine Similarity
        self.index = nmslib.init(method='hnsw', space='cosinesimil') if not self.index else self.index
        self.index.addDataPointBatch(vectors, ids)
        self.index.createIndex({'post': 2})

    def _parse_result(self, pair):
        return OrderedDict([(self.id2word[k], 1 - v) for k, v in zip(*pair)])


if __name__ == '__main__':
    vq = VecQuery()
    vq.createIndex(['a', 'b'], [[1, 2], [3, 4]])
    print(vq.query([[1, 2]]))
    print(vq.id2word, vq.word2id)
    vq.createIndex(['a', 'c'], [[1, 2], [3, 3]])

    print(vq.query([[1, 2], [1, 2]]))
    print(vq.id2word, vq.word2id)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : iWork.
# @File         : delta
# @Time         : 2019-06-13 22:15
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

# https://www.cnblogs.com/sunshineyang/p/6818834.html
import datetime
from dateutil.relativedelta import relativedelta

d1 = datetime.datetime.today()
d2 = d1 + relativedelta(months=1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : timer
# @Time         : 2019-06-20 11:26
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  :
import time
from contextlib import contextmanager

from .. import cprint


@contextmanager
def timer(task_name="timer"):
    # a timer cm from https://www.kaggle.com/lopuhin/mercari-golf-0-3875-cv-in-75-loc-1900-s
    cprint(">>> {} started".format(task_name))
    t0 = time.time()
    yield
    cprint(">>> {} done in {:.0f} seconds".format(task_name, time.time() - t0))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : iWork.
# @File         : Time
# @Time         : 2019-06-13 22:17
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 
import time
import numpy as np
import dateutil
import pendulum
from pytz import timezone
"""

from pytz import timezone
cst_tz = timezone('Asia/Shanghai') 
import dateutil
from dateutil.parser import parser 
dateutil.tz.tzstr('Asia/Shanghai') # 时间解析
pd.to_datetime(ts, errors='coerce', infer_datetime_format=True) # pandas调用dateutil的时间解析

"""

class DateTime(object):
    timezoneMap = {"CST": "Asia/Shanghai"}

    def __init__(self):
        self.tz = "Asia/Shanghai"

    def time_to_pdate(self, timestamp):
        return pendulum.from_timestamp(timestamp, self.tz)

    def time_to_date_string(self, timestamp):
        return self.time_to_pdate(timestamp).to_date_string()

    def time_to_datetime_string(self, timestamp):
        return self.time_to_pdate(timestamp).to_datetime_string()

    @property
    def nowTime(self):
        return self.nowDate.timestamp()

    @property
    def nowDate(self):
        return pendulum.now(self.tz)

    @property
    def timezone(self):
        return time.strftime("%Z", time.localtime())  # 获取当前时区


if __name__ == '__main__':
    dt = DateTime()
    print(dt.nowDate)
    print(dt.nowTime)
    print(dt.time_to_date_string(1551843001))
    print(dt.time_to_datetime_string(1551843001))

    from dateutil.relativedelta import relativedelta
    import datetime

    today = datetime.date.today()  # 今天

    yesterday = today - datetime.timedelta(days=1)  # 昨天
    Lastweek = today - datetime.timedelta(days=7)  # 上周
    Nearly_month = today - datetime.timedelta(days=30)  # 前30天
    Last_month = datetime.date.today() - relativedelta(months=1)  # 上月
    Last_year = datetime.date.today() - relativedelta(months=12)  # 去年



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'WordVec2BOW'
__author__ = 'JieYuan'
__mtime__ = '19-3-6'
"""
from sklearn.feature_extraction.text import CountVectorizer


class WordVec2BOW(object):

    def __init__(self, wv):
        self.wv = wv  # wv = gensim.models.KeyedVectors.load_word2vec_format('./title_words.vector.bin', binary=True)
        self.cv = CountVectorizer(tokenizer=lambda x: x, lowercase=False)
        self.cv.fit([self.wv.index2word])  # wv词库务必包含corpus, 可改写为wv词库与corpus并集
        self.vocabulary = self.cv.vocabulary_

    def transform(self, corpus):
        mat = self.cv.transform(corpus).tolil().astype(float)
        for i in range(len(corpus)):
            cosine_dict = self.get_cosine_dict(corpus[i])
            word_idx = [self.vocabulary.get(w) for w in cosine_dict]
            word_value = list(cosine_dict.values())
            mat[i, word_idx] = word_value
        return mat

    def get_cosine_dict(self, sent, topn=10):
        """
        从词的角度出发考虑的，最后的效果非常好，就是怎么样从词的向量得到句子的向量。
        首先选出一个词库，比如说10万个词，然后用w2v跑出所有词的向量，然后对于每一个句子，
        构造一个10万维的向量，向量的每一维是该维对应的词和该句子中每一个词的相似度的最大值。
        """
        dic = {}
        for w in sent:
            if w in self.wv:
                _ = self.wv.similar_by_word(w, topn)
                dic.update({k: max(dic.get(k, 0), v) for k, v in _ if k not in sent})
        return dic



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : Callbacks
# @Time         : 2019-07-10 17:27
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 
from tensorflow.python.keras.callbacks import *


# RemoteMonitor, ModelCheckpoint, LearningRateScheduler, EarlyStopping, ReduceLROnPlateau

class Callbacks(object):
    """
    https://blog.csdn.net/dayuqi/article/details/85090353
    """

    def __init__(self, early_stopping_epochs=4, monitor='val_loss', save_weights_only=False,
                 weightsfile="best_model_weights.hdf5"):
        self.monitor = monitor
        self.weightsfile = weightsfile
        self.save_weights_only = save_weights_only  # 默认保存模型
        self.early_stopping_epochs = early_stopping_epochs
        self.callbacks = [self.checkpointer, self.lr_reducing, self.early_stopping]

    def tensorboard(self, log_dir='logs'):
        """tensorboard --logdir=/full_path_to_your_logs"""
        TensorBoard(log_dir,
                    histogram_freq=0,
                    write_graph=True,
                    write_images=False,
                    update_freq='epoch',
                    profile_batch=2,
                    embeddings_freq=0,
                    embeddings_metadata=None)

    @property
    def checkpointer(self):
        """
        save_freq=128相当于一步判断一次（按样本数更新保存最优权重）
        """
        # filepath="weights-improvement-{epoch}-{val_acc:.2f}.hdf5" # 多个check point
        return ModelCheckpoint(filepath=self.weightsfile,
                               monitor=self.monitor,
                               save_best_only=True,
                               save_weights_only=self.save_weights_only,
                               save_freq='epoch',
                               verbose=1)

    @property
    def lr_reducing(self):
        """动态学习率"""
        # new_lr = old_lr * self.factor
        annealer = ReduceLROnPlateau(factor=0.9, patience=2, verbose=1, min_lr=0.0001)
        # annealer = LearningRateScheduler(lambda x: min(0.01 * 0.9 ** x, 0.001), verbose=1)
        return annealer

    @property
    def early_stopping(self):
        """patience次没提高就停止"""
        return EarlyStopping(patience=self.early_stopping_epochs,
                             restore_best_weights=True,
                             verbose=1)



from sanic import Sanic
from sanic import response
from jinja2 import Environment, PackageLoader, select_autoescape

import sys

# Enabling async template execution which allows you to take advantage
# of newer Python features requires Python 3.6 or later.
enable_async = sys.version_info >= (3, 6)

app = Sanic(__name__)

# Load the template environment with async support
template_env = Environment(
    loader=PackageLoader('jinja_example', 'templates', 'index'),  # 所需文件
    autoescape=select_autoescape(['html', 'xml']),
    enable_async=enable_async
)

# Load the template from file
template = template_env.get_template("example_template.html")


@app.route('/')
async def test(request):
    rendered_template = await template.render_async(
        knights='YuanJie')
    return response.html(rendered_template)


app.run(host="0.0.0.0", port=8000, debug=True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : lazy_import
# @Time         : 2019-07-12 10:30
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


class LazyImport:
    def __init__(self, module_name):
        self.module_name = module_name
        self.module = None

    def __getattr__(self, name):
        if self.module is None:
            self.module = __import__(self.module_name)
        return getattr(self.module, name)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'demo'
__author__ = 'JieYuan'
__mtime__ = '2019-04-30'
"""

import requests


def request(url, params=None):
    try:
        # headers：伪装spider
        headers = {'user-agent': 'Mozilla/5.0'}
        r = requests.get(url, params, headers=headers)
        r.raise_for_status()  # 申请返回的状态码: 200为成功
        r.encoding = r.apparent_encoding
        print(r.text.split()[:32])
        return r

    except Exception as e:
        print('爬取失败：%s' % e)


if __name__ == '__main__':
    request('https://www.baidu.com/s?wd=周杰伦')
    request('https://www.baidu.com/s', {'wd': '周杰伦'})



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'fire'
__author__ = 'JieYuan'
__mtime__ = '19-1-10'
"""

import fire


class Config(object):
    """默认参数"""
    x = 1
    y = 2
    z = 3


opt = Config()


def gen(**kwargs):
    for item in kwargs.items():
        setattr(opt, *item)

    _ = opt.x + opt.y + opt.z  # 操作逻辑
    return _


if __name__ == '__main__':
    fire.Fire()  # python ./fire_args.py gen --x 100
    # fire.Fire() # python fire_args.py --x 10000



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : TextRCNNVariant
# @Time         : 2019-06-24 17:22
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

from .BaseModel import BaseModel
from tensorflow.python.keras import backend as K
from tensorflow.python.keras import Input, Model
from tensorflow.python.keras.layers import Dense, Embedding, Lambda, Concatenate
from tensorflow.python.keras.layers import Conv1D, GlobalMaxPooling1D, GlobalAveragePooling1D
from tensorflow.python.keras.layers import Bidirectional, LSTM, CuDNNLSTM, GRU, CuDNNGRU


class TextRCNNVariant(BaseModel):
    """Variant of RCNN.
        Base on structure of RCNN, we do some improvement:
        1. Ignore the shift for left/right context.
        2. Use Bidirectional LSTM/GRU to encode context.
        3. Use Multi-CNN to represent the semantic vectors.
        4. Use ReLU instead of Tanh.
        5. Use both AveragePooling and MaxPooling.
    """

    def __init__(self, max_tokens, maxlen=128, embedding_size=None, num_class=1, weights=None,
                 kernel_size_list=range(1, 5)):
        """
        :param rnn: LSTM, CuDNNLSTM, GRU, CuDNNGRU
        """
        super().__init__(max_tokens, maxlen, embedding_size, num_class, weights)
        self.kernel_size_list = kernel_size_list

    def get_model(self):
        input = Input((self.maxlen,))

        embedding = Embedding(self.max_tokens, self.embedding_size, input_length=self.maxlen)(input)

        x_context = Bidirectional(CuDNNLSTM(128, return_sequences=True))(embedding)
        x = Concatenate()([embedding, x_context])

        convs = []
        for kernel_size in self.kernel_size_list:
            conv = Conv1D(128, kernel_size, activation='relu')(x)
            convs.append(GlobalMaxPooling1D()(conv))
            convs.append(GlobalAveragePooling1D()(conv))

        # poolings = [GlobalAveragePooling1D()(conv) for conv in convs] + \
        #            [GlobalMaxPooling1D()(conv) for conv in convs]
        x = Concatenate()(convs)

        output = Dense(self.num_class, activation=self.last_activation)(x)
        model = Model(inputs=input, outputs=output)
        return model



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : demo
# @Time         : 2019-09-03 21:55
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

import deepctr




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : DF2FFM
# @Time         : 2019-08-26 20:18
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  :

import json
from tqdm import tqdm
from csv import DictReader


def csv2libffm(input, fieldnames=None, cols_cat=None, cols_num=None, cols_multi_cat=None, label='label', sep=',',
               output='train.libffm', features_map='features.json'):
    global field_features
    field_features = {}
    dialect_map = {',': 'excel', '\t': 'excel-tab'}
    with open(output, 'a') as output:
        with open(input) as input:
            for row in tqdm(DictReader(input, fieldnames, dialect=dialect_map[sep]), desc="Transform ..."):
                line = []

                # 类别型
                cols_cat = cols_cat if cols_cat else [i for i in row.keys() if i != label]  # 默认全部为 类别型
                for idx, field in enumerate(cols_cat, 1):
                    feature = row[field]
                    ff = field + '______' + feature if feature else 'NA'  # 类别值就是一个 bin
                    fnum = field_features.setdefault(ff, len(field_features) + 1)

                    line.append(f'{idx}:{fnum}:1')

                # 多类别型： TODO: cols_multi_cat

                # 数值型
                if cols_num is not None:
                    for idx, field in enumerate(cols_num, idx + 1):
                        feature = row[field]
                        ff = field  # 数值当作一个 bin
                        fnum = field_features.setdefault(ff, len(field_features) + 1)

                        if feature != '' and feature != '0':
                            line.append(f'{idx}:{fnum}:{feature}')

                output.write(f"{row.get(label, '-1')} {' '.join(line)}\n")

    with open(features_map, 'w') as f:
        return json.dump(field_features, f)

# ########################### Lets build some data and test ############################
#
# train, y = make_classification(n_samples=10, n_features=5, n_informative=2, n_redundant=2, n_classes=2, random_state=42)
#
# train = pd.DataFrame(train, columns=['int1', 'int2', 'int3', 's1', 's2'])
# train['int1'] = train['int1'].map(int)
# train['int2'] = train['int2'].map(int)
# train['int3'] = train['int3'].map(int)
# train['s1'] = round(np.log(abs(train['s1'] + 1))).map(str)
# train['s2'] = round(np.log(abs(train['s2'] + 1))).map(str)
# train['clicked'] = y
#
# ffm_train = FFMFormatPandas()
# ffm_train_data = ffm_train.fit_transform(train, y='clicked')
# print('Base data')
# print(train[0:10])
# print('FFM data')
# print(ffm_train_data[0:10])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'row_feats'
__author__ = 'JieYuan'
__mtime__ = '19-2-25'
"""
from .Funcs import Funcs
import pandas as pd


class RowFeats(object):

    def __init__(self):
        pass

    def transform(self, df: pd.DataFrame, include=None, exclude=None):

        if not exclude is None and include is None:
            if exclude:
                df = df[exclude]
            else:
                df = df[[col for col in df.columns if col not in exclude]]
        else:
            df = df.copy()

        num_funcs = Funcs().num_funcs
        func_names = ['min', 'mean', 'median', 'max', 'sum', 'std', 'sem', 'skew', 'kurt']
        func_names_plus = ['q1', 'q3', 'iqr', 'kurt', 'cv', 'p2p']

        func = lambda x: [x.__getattr__(f)() for f in func_names] + [f(x) for f in num_funcs]
        df = df.apply(func, 1, result_type='expand')
        df.columns = func_names + func_names_plus
        return df



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'GaussRankScaler'
__author__ = 'JieYuan'
__mtime__ = '19-3-11'
"""

import scipy


class GaussRankScaler():
    """
    https://www.kaggle.com/mathormad/knowledge-distillation-with-nn-rankgauss
    https://blog.csdn.net/qq_33139511/article/details/79721590
    """

    def __init__(self):
        self.epsilon = 1e-9
        self.lower = -1 + self.epsilon
        self.upper = 1 - self.epsilon
        self.range = self.upper - self.lower

    def fit_transform(self, X):
        i = np.argsort(X, axis=0)
        j = np.argsort(i, axis=0)

        assert (j.min() == 0).all()
        assert (j.max() == len(j) - 1).all()

        j_range = len(j) - 1
        self.divider = j_range / self.range

        transformed = j / self.divider
        transformed = transformed - self.upper
        transformed = scipy.special.erfinv(transformed)
        ############
        # transformed = transformed - np.mean(transformed)

        return transformed



# -*- coding: utf-8 -*-

"""
__title__ = 'twv'
__author__ = 'JieYuan'
__mtime__ = '18-11-13'
"""

from tqdm import tqdm
import pymongo

from concurrent.futures import ThreadPoolExecutor
from scipy.spatial.distance import pdist


class TencentWord2Vec(object):

    def __init__(self):
        self.db = pymongo.MongoClient().ChineseEmbedding.TencentWord2Vec

    def __getitem__(self, items):
        if isinstance(items, str):
            return self.get_vector([items])[0]
        else:
            return self.get_vector(items)

    def cosine(self, w1, w2):
        return 1 - pdist(self.get_vector((w1, w2)), 'cosine')[0]

    def get_vector(self, words):
        """
        from sklearn.metrics.pairwise import cosine_similarity
        >>> a=[[1,3,2],[2,2,1]]
        >>> cosine_similarity(a)
        """
        with ThreadPoolExecutor(8 if len(words) > 8 else len(words)) as pool:
            vecs = pool.map(self.__func, words)
        return list(vecs)

    def __insert_word2vec(self):

        self.db.delete_many({})
        for line in self.__reader():
            _ = self.db.insert_one(line)

    def __func(self, word):
        return self.db.find_one({'word': word})['vector']

    def __reader(self):
        with open("/home/yuanjie/下载/Tencent_AILab_ChineseEmbedding/Tencent_AILab_ChineseEmbedding.txt") as f:
            for idx, line in tqdm(enumerate(f), 'Loading ...'):
                ws = line.strip().split(' ')
                if idx:
                    yield {'word': ws[0], 'vector': [float(i) for i in ws[1:]]}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : TextRNN
# @Time         : 2019-06-24 11:23
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

from .BaseModel import BaseModel
from tensorflow.python.keras import Input, Model
from tensorflow.python.keras.layers import Dense, LSTM, CuDNNLSTM, GRU, CuDNNGRU


class TextRNN(BaseModel):
    def __init__(self, max_tokens, maxlen=128, embedding_size=None, num_class=1, weights=None, rnn=CuDNNGRU):
        """
        :param rnn: LSTM, CuDNNLSTM, GRU, CuDNNGRU
        """
        super().__init__(max_tokens, maxlen, embedding_size, num_class, weights)
        self._RNN = rnn

    def get_model(self):
        input = Input(self.maxlen)
        embedding = self.embedding_layer(input)
        x = self._RNN(128)(embedding)  # LSTM or GRU
        output = Dense(self.num_class, activation=self.last_activation)(x)
        model = Model(inputs=input, outputs=output)
        return model



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = '__init__.py'
__author__ = 'JieYuan'
__mtime__ = '19-1-3'
"""
from .embedding_tree import EmbeddingTrees
from .GensimDoc2Vec import GensimDoc2Vec



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : EntityEmbedding
# @Time         : 2019-07-15 10:30
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


# class EntityEmbedding(Model):
#
#     def __init__(self, X_train, y_train, X_val, y_val):
#         super().__init__()
#         self.epochs = 10
#         self.checkpointer = ModelCheckpoint(filepath="best_model_weights.hdf5", verbose=1, save_best_only=True)
#         self.max_log_y = max(numpy.max(numpy.log(y_train)), numpy.max(numpy.log(y_val)))
#         self.__build_keras_model()
#         self.fit(X_train, y_train, X_val, y_val)
#
#     def preprocessing(self, X):
#         X_list = split_features(X)
#         return X_list
#
#     def __build_keras_model(self):
#         input_store = Input(shape=(1,))
#         output_store = Embedding(1115, 10, name='store_embedding')(input_store)
#         output_store = Reshape(target_shape=(10,))(output_store)
#
#         input_dow = Input(shape=(1,))
#         output_dow = Embedding(7, 6, name='dow_embedding')(input_dow)
#         output_dow = Reshape(target_shape=(6,))(output_dow)
#
#         input_promo = Input(shape=(1,))
#         output_promo = Dense(1)(input_promo)
#
#         input_year = Input(shape=(1,))
#         output_year = Embedding(3, 2, name='year_embedding')(input_year)
#         output_year = Reshape(target_shape=(2,))(output_year)
#
#         input_month = Input(shape=(1,))
#         output_month = Embedding(12, 6, name='month_embedding')(input_month)
#         output_month = Reshape(target_shape=(6,))(output_month)
#
#         input_day = Input(shape=(1,))
#         output_day = Embedding(31, 10, name='day_embedding')(input_day)
#         output_day = Reshape(target_shape=(10,))(output_day)
#
#         input_germanstate = Input(shape=(1,))
#         output_germanstate = Embedding(12, 6, name='state_embedding')(input_germanstate)
#         output_germanstate = Reshape(target_shape=(6,))(output_germanstate)
#
#         input_model = [input_store, input_dow, input_promo,
#                        input_year, input_month, input_day, input_germanstate]
#
#         output_embeddings = [output_store, output_dow, output_promo,
#                              output_year, output_month, output_day, output_germanstate]
#
#         output_model = Concatenate()(output_embeddings)
#         output_model = Dense(1000, kernel_initializer="uniform")(output_model)
#         output_model = Activation('relu')(output_model)
#         output_model = Dense(500, kernel_initializer="uniform")(output_model)
#         output_model = Activation('relu')(output_model)
#         output_model = Dense(1)(output_model)
#         output_model = Activation('sigmoid')(output_model)
#
#         self.model = KerasModel(inputs=input_model, outputs=output_model)
#
#         self.model.compile(loss='mean_absolute_error', optimizer='adam')
#
#     def _val_for_fit(self, val):
#         val = numpy.log(val) / self.max_log_y
#         return val
#
#     def _val_for_pred(self, val):
#         return numpy.exp(val * self.max_log_y)
#
#     def fit(self, X_train, y_train, X_val, y_val):
#         self.model.fit(self.preprocessing(X_train), self._val_for_fit(y_train),
#                        validation_data=(self.preprocessing(X_val), self._val_for_fit(y_val)),
#                        epochs=self.epochs, batch_size=128,
#                        # callbacks=[self.checkpointer],
#                        )
#         # self.model.load_weights('best_model_weights.hdf5')
#         print("Result on validation data: ", self.evaluate(X_val, y_val))
#
#     def guess(self, features):
#         features = self.preprocessing(features)
#         result = self.model.predict(features).flatten()
#         return self._val_for_pred(result)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'SparkInit'
__author__ = 'JieYuan'
__mtime__ = '2019-05-13'
"""

from pyspark.sql import *


class SparkInit(object):
    """
    from pyspark.sql.types import *
    import pyspark.sql.functions as F
    """

    def __init__(self, load_mmlspark=False):
        """
        sc, spark = SparkInit()()
        """
        if load_mmlspark:
            self.spark = (SparkSession.builder
                          .appName("Yuanjie")
                          .config('log4j.rootCategory', "WARN")
                          .config("spark.jars.packages", "Azure:mmlspark:0.17")
                          .enableHiveSupport()
                          .getOrCreate())
        else:
            self.spark = (SparkSession.builder
                          .appName("Yuanjie")
                          .config('log4j.rootCategory', "WARN")
                          .enableHiveSupport()
                          .getOrCreate())

        self.sc = self.spark.sparkContext
        print('Spark Version: %s' % self.spark.version)

    def __call__(self, *args, **kwargs):
        return self.sc, self.spark



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = '__init__.py'
__author__ = 'JieYuan'
__mtime__ = '19-3-12'
"""
from .timer import timer
from .time_ import timestamp2str, exponential_decay



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : Capsule
# @Time         : 2019-07-10 16:39
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


from tensorflow.python.keras import backend as K
from tensorflow.python.keras import activations
from tensorflow.python.keras.models import Layer


def squash(x, axis=-1):
    s_squared_norm = K.sum(K.square(x), axis, keepdims=True) + K.epsilon()
    scale = K.sqrt(s_squared_norm) / (0.5 + s_squared_norm)
    return scale * x


# define our own softmax function instead of K.softmax
def softmax(x, axis=-1):
    ex = K.exp(x - K.max(x, axis=axis, keepdims=True))
    return ex / K.sum(ex, axis=axis, keepdims=True)


# A Capsule Implement with Pure Keras
class Capsule(Layer):
    def __init__(self, num_capsule, dim_capsule, routings=3, share_weights=True, activation='squash', **kwargs):
        super(Capsule, self).__init__(**kwargs)
        self.num_capsule = num_capsule
        self.dim_capsule = dim_capsule
        self.routings = routings
        self.share_weights = share_weights
        if activation == 'squash':
            self.activation = squash
        else:
            self.activation = activations.get(activation)

    def build(self, input_shape):
        super(Capsule, self).build(input_shape)
        input_dim_capsule = input_shape[-1]
        if self.share_weights:
            self.W = self.add_weight(name='capsule_kernel',
                                     shape=(1, input_dim_capsule,
                                            self.num_capsule * self.dim_capsule),
                                     initializer='glorot_uniform',
                                     trainable=True)
        else:
            input_num_capsule = input_shape[-2]
            self.W = self.add_weight(name='capsule_kernel',
                                     shape=(input_num_capsule,
                                            input_dim_capsule,
                                            self.num_capsule * self.dim_capsule),
                                     initializer='glorot_uniform',
                                     trainable=True)

    def call(self, u_vecs, **kwargs):
        if self.share_weights:
            u_hat_vecs = K.conv1d(u_vecs, self.W)
        else:
            u_hat_vecs = K.local_conv1d(u_vecs, self.W, [1], [1])

        batch_size = K.shape(u_vecs)[0]
        input_num_capsule = K.shape(u_vecs)[1]
        u_hat_vecs = K.reshape(u_hat_vecs, (batch_size, input_num_capsule,
                                            self.num_capsule, self.dim_capsule))
        u_hat_vecs = K.permute_dimensions(u_hat_vecs, (0, 2, 1, 3))
        # final u_hat_vecs.shape = [None, num_capsule, input_num_capsule, dim_capsule]

        b = K.zeros_like(u_hat_vecs[:, :, :, 0])  # shape = [None, num_capsule, input_num_capsule]
        for i in range(self.routings):
            c = softmax(b, 1)
            o = K.batch_dot(c, u_hat_vecs, [2, 2])
            if K.backend() == 'theano':
                o = K.sum(o, axis=1)
            if i < self.routings - 1:
                o = K.l2_normalize(o, -1)
                b = K.batch_dot(o, u_hat_vecs, [2, 3])
                if K.backend() == 'theano':
                    b = K.sum(b, axis=1)

        return self.activation(o)

    def compute_output_shape(self, input_shape):
        return (None, self.num_capsule, self.dim_capsule)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : DF2FM
# @Time         : 2019-08-26 20:16
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 
import numpy as np
import pandas as pd
from sklearn.datasets import make_classification
from sklearn.datasets import dump_svmlight_file, load_svmlight_file

train, y = make_classification(n_samples=10, n_features=5, n_informative=2, n_redundant=2, n_classes=2, random_state=42)

train = pd.DataFrame(train, columns=['int1', 'int2', 'int3', 's1', 's2'])
train['int1'] = train['int1'].map(int)
train['int2'] = train['int2'].map(int)
train['int3'] = train['int3'].map(int)
train['s1'] = round(np.log(abs(train['s1'] + 1))).map(str)
train['s2'] = round(np.log(abs(train['s2'] + 1))).map(str)
train['clicked'] = y

dump_svmlight_file(pd.get_dummies(train), y, 'svm_output.libsvm', zero_based=False)  # 从1开始编码
dump_svmlight_file(pd.get_dummies(train[['s1', 's2']]), y, 'svm_output.libsvm', zero_based=False)  # 从1开始编码



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = 'kg'
__author__ = 'JieYuan'
__mtime__ = '19-3-25'
"""

import json
import urllib

import requests


class BaiduPost(object):
    """
    知识图谱： https://aip.baidubce.com/rpc/2.0/kg/v1/cognitive/entity_annotation
    智能春联： https://aip.baidubce.com/rpc/2.0/nlp/v1/couplets
        { "text": "百度", "index": 0}
    智能写诗: https://aip.baidubce.com/rpc/2.0/nlp/v1/poem
        { "text": "百度", "index": 0} # index默认为数值为0，即第一首诗。每换一次，数值加1即可，一定数量后会返回之前的作诗结果

    """

    def __init__(self, api_key='B0cphM43tmxUyR4YcGMvCn1X', secret_key='SjRpd3Fg6aaX9B5KQD796IDHgvFMrURb'):
        self.access_token = self._get_access_token(api_key, secret_key)

    def predict(self, input_text, url):
        url = url + '?charset=UTF-8&access_token=' + self.access_token

        # the input is json format
        # input_text = {'data': text}
        r = requests.post(url, json=input_text)
        return r.json()

    def _get_access_token(self, api_key, secret_key):
        host = f'https://aip.baidubce.com/oauth/2.0/token?grant_type=client_credentials&client_id={api_key}&client_secret={secret_key}'
        request = urllib.request.Request(host)
        request.add_header('Content-Type', 'application/json; charset=UTF-8')

        response = urllib.request.urlopen(request)
        content = response.read()
        return json.loads(content)['access_token']


if __name__ == '__main__':
    from pprint import pprint

    api = BaiduPost()
    pprint(api.predict({'data': '周杰伦'},
                       'https://aip.baidubce.com/rpc/2.0/kg/v1/cognitive/entity_annotation'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : qps
# @Time         : 2019-08-29 11:22
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

import requests
from locust import HttpLocust,TaskSet,task


class MyBlogs(TaskSet):
    # 访问我的博客首页
    @task(1)
    def get_blog(self):
        # 定义请求头
        header = {"User-Agent": "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/69.0.3497.100 Safari/537.36"}

        req = self.client.get("/imyalost",  headers=header, verify=False)
        if req.status_code == 200:
            print("success")
        else:
            print("fails")

class websitUser(HttpLocust):
    task_set = MyBlogs
    min_wait = 3000  # 单位为毫秒
    max_wait = 6000  # 单位为毫秒

if __name__ == "__main__":
    import os
    os.system("locust -f qps.py --host=https://www.cnblogs.com")